import json
from dataclasses import dataclass
from datetime import date
from pathlib import Path
from typing import TYPE_CHECKING, Any, Callable, Optional, Protocol, runtime_checkable

from numpy.typing import ArrayLike
from pydantic import BaseModel

from seb.interfaces.language import Language

if TYPE_CHECKING:
    from .task import Task


@runtime_checkable
class Encoder(Protocol):
    """
    Interface which all models must implement.
    """

    def encode(
        self,
        sentences: list[str],
        *,
        task: Optional["Task"] = None,
        batch_size: int = 32,
        **kwargs: Any,
    ) -> ArrayLike:
        """Returns a list of embeddings for the given sentences.

        Args:
            sentences: List of sentences to encode
            task: The task to encode for. This allows the model to encode differently for different tasks. Will always be given but does not need
                to be used.
            batch_size: Batch size for the encoding
            kwargs: arguments to pass to the models encode method

        Returns:
            Embeddings for the given documents
        """
        ...


class ModelMeta(BaseModel):
    """
    The metadata object for a model. This includes information such as the name, description, languages, etc.
    """

    name: str
    description: Optional[str] = None
    huggingface_name: Optional[str] = None
    reference: Optional[str] = None
    languages: list[Language] = []
    open_source: bool = False
    embedding_size: Optional[int] = None
    model_type: Optional[str] = None
    release_date: Optional[date] = None

    def get_path_name(self) -> str:
        if self.huggingface_name is None:
            return self._name_to_path(self.name)
        return self._name_to_path(self.huggingface_name)

    @staticmethod
    def _name_to_path(name: str) -> str:
        return name.replace("/", "__").replace(" ", "_")

    def get_huggingface_url(self) -> str:
        if self.huggingface_name is None:
            raise ValueError("This model does not have an associated huggingface name.")
        return f"https://huggingface.co/{self.huggingface_name}"

    def to_disk(self, path: Path) -> None:
        with path.open("w") as f:
            f.write(self.model_dump_json())

    @classmethod
    def from_disk(cls, path: Path) -> "ModelMeta":
        with path.open() as f:
            model_meta = json.load(f)
        return cls(**model_meta)


@dataclass
class LazyLoadEncoder(Encoder):
    """Encoder object, which lazy loads the model on the first call to encode()"""

    loader: Callable[[], Encoder]
    _model: Optional[Encoder] = None

    @property
    def model(self) -> Encoder:
        """
        Dynimically load the model.
        """
        if self._model is None:
            self._model = self.loader()
        return self._model

    def encode(
        self,
        sentences: list[str],
        *,
        task: Optional["Task"] = None,
        batch_size: int = 32,
        **kwargs: Any,
    ) -> ArrayLike:
        """
        Returns a list of embeddings for the given sentences.
        Args:
            sentences: List of sentences to encode
            task: The task to encode for. This allows the model to encode differently for different tasks. Will always be given but does not need
                to be used.
            batch_size: Batch size for the encoding
            kwargs: arguments to pass to the models encode method

        Returns:
            Embeddings for the given documents
        """
        return self.model.encode(sentences, batch_size=batch_size, task=task, **kwargs)

    def encode_queries(self, queries: list[str], batch_size: int, **kwargs):  # noqa
        try:
            return self.model.encode_queries(queries, batch_size=batch_size, **kwargs)  # type: ignore
        except AttributeError:
            return self.encode(queries, task=None, batch_size=batch_size, **kwargs)

    def encode_corpus(self, corpus: list[dict[str, str]], batch_size: int, **kwargs):  # noqa
        try:
            return self.model.encode_corpus(corpus, batch_size=batch_size, **kwargs)  # type: ignore
        except AttributeError:
            sep = " "
            if isinstance(corpus, dict):
                sentences = [
                    (corpus["title"][i] + sep + corpus["text"][i]).strip() if "title" in corpus else corpus["text"][i].strip()  # type: ignore
                    for i in range(len(corpus["text"]))  # type: ignore
                ]
            else:
                sentences = [(doc["title"] + sep + doc["text"]).strip() if "title" in doc else doc["text"].strip() for doc in corpus]
            return self.encode(sentences, task=None, batch_size=batch_size, **kwargs)


@dataclass
class SebModel:
    """
    An embedding model as implemented in SEB. It notably dynamically loads models (such that models are not loaded when a cache is hit)
    and includes metadata pertaining to the specific model.
    """

    meta: ModelMeta
    encoder: Encoder

    @property
    def number_of_parameters(self) -> Optional[int]:
        """
        Returns the number of parameters in the model.
        """
        if hasattr(self.encoder, "num_parameters"):
            return sum(p.numel() for p in self.model.parameters() if p.requires_grad)  # type: ignore
        return None



import logging
import platform
import time
from datetime import datetime
from pathlib import Path
from typing import Callable, Optional, Union

import numpy as np
import psutil
import torch

from seb.interfaces.language import languages_in_seb
from seb.interfaces.model import Encoder
from seb.interfaces.task import DescriptiveDatasetStats, Task
from seb.result_dataclasses import TaskResult

logger = logging.getLogger(__name__)

TOKENS_IN_UGLY_DUCKLING = 3591


class CPUSpeedTask(Task):
    reference = "NA"
    version = "0.0.1"
    task_type = "Speed"
    languages = languages_in_seb
    main_score = "Inference speed (seconds)"
    domain = ["fiction"]  # noqa
    name = "Speed (CPU)"
    description = "Time taken to encode the text 'The Ugly Duckling' split by paragraphs on a CPU."
    device = "cpu"
    _dataset: Optional[list[str]] = None

    def load_dataset(self) -> list[str]:
        file_path = Path(__file__).parent / "the_ugly_duckling.txt"
        with file_path.open("r") as f:
            text = f.read()
        return text.split("\n\n")

    @property
    def dataset(self) -> list[str]:
        if self._dataset is None:
            self._dataset = self.load_dataset()
        return self._dataset

    def get_descriptive_stats(self) -> DescriptiveDatasetStats:
        dataset = self.load_dataset()
        lengths = np.array([len(x) for x in dataset])
        return DescriptiveDatasetStats(
            mean_document_length=float(np.mean(lengths)),
            std_document_length=float(np.std(lengths)),
            num_documents=len(dataset),
        )

    def get_time_taken(self, model: Encoder) -> float:
        dataset = self.load_dataset()
        start = time.time()
        with torch.no_grad():
            model.encode(dataset, device=self.device, task=self)
        time_taken = time.time() - start
        return time_taken

    def evaluate(self, model: Encoder) -> TaskResult:  # type: ignore
        model.encode(["encode this"])  # ensure model is loaded

        has_to_method = hasattr(model._model, "to") and isinstance(model._model.to, Callable)  # type: ignore
        if has_to_method:
            model._model = model._model.to(self.device)  # type: ignore

        run_inference = not (self.device == "cuda" and not has_to_method)
        if run_inference:
            time_taken = self.get_time_taken(model)
        else:
            time_taken = np.nan

        scores: dict[str, Union[str, float]] = {
            self.main_score: time_taken,
            "words pr. second": TOKENS_IN_UGLY_DUCKLING / time_taken,
            **self.get_system_info(),
        }

        return TaskResult(
            task_name=self.name,
            task_description=self.description,
            task_version=self.version,
            scores={Language: scores for Language in self.languages},  # type: ignore
            time_of_run=datetime.now(),
            main_score=self.main_score,
        )

    def get_system_info(self) -> dict[str, str]:
        """
        Returns a dictionary with system information.
        """
        info = {}
        info["platform"] = platform.system()
        info["platform-release"] = platform.release()
        info["platform-version"] = platform.version()
        info["architecture"] = platform.machine()
        info["processor"] = platform.processor()
        info["ram"] = str(round(psutil.virtual_memory().total / (1024.0**3))) + " GB"
        info["Physical cores"] = psutil.cpu_count(logical=False)
        info["Total cores"] = psutil.cpu_count(logical=True)
        return info


class GPUSpeedTask(CPUSpeedTask):
    name = "Speed (GPU)"
    description = "Time taken to encode the text 'The Ugly Duckling' split by paragraphs on a GPU."
    device: str = "cuda"



from typing import Any

from mteb.abstasks import AbsTaskClassification


class NorwegianParliamentClassification(AbsTaskClassification):
    # this changes the description of the tasks but otherwise is the same as the task in the MTEB benchmark
    # once we have collected a few MTEB tasks not in the MTEB benchmark we can add them back to the benchmark.
    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "NorwegianParliament",
            "hf_hub_name": "NbAiLab/norwegian_parliament",
            "description": "Norwegian parliament speeches annotated with the party of the speaker (`Sosialistisk Venstreparti` vs `Fremskrittspartiet`)",
            "reference": "https://huggingface.co/datasets/NbAiLab/norwegian_parliament",
            "type": "Classification",
            "category": "s2s",
            "eval_splits": ["test", "validation"],
            "eval_langs": ["nb"],  # assumed to be bokmål
            "main_score": "accuracy",
            "n_experiments": 10,
            "samples_per_label": 16,
            "revision": "f7393532774c66312378d30b197610b43d751972",
        }



from typing import Optional

import pandas as pd
from rich.console import Console
from rich.table import Table

import seb
from seb.interfaces.language import Language


def get_main_score(task: seb.TaskResult, langs: Optional[list[Language]]) -> float:
    if langs is None:  # noqa
        _langs = task.languages
    else:
        _langs = set(langs) & set(task.languages)
    return task.get_main_score(_langs) * 100


def open_source_to_string(open_source: bool) -> str:
    return "✓" if open_source else "✗"


def benchmark_result_to_row(
    result: seb.BenchmarkResults,
    langs: Optional[list[str]],
) -> pd.DataFrame:
    mdl_name = result.meta.name
    # sort by task name
    task_results = result.task_results
    sorted_tasks = sorted(task_results, key=lambda t: t.task_name)
    task_names = [t.task_name for t in sorted_tasks]
    scores = [get_main_score(t, langs) for t in sorted_tasks]  # type: ignore

    df = pd.DataFrame([scores], columns=task_names, index=[mdl_name])  # type: ignore
    df["Average Score"] = result.get_main_score()  # type: ignore
    df["Open Source"] = open_source_to_string(result.meta.open_source)
    df["Embedding Size"] = result.meta.embedding_size
    return df


def convert_to_table(
    results: list[seb.BenchmarkResults],
    langs: Optional[list[str]],
) -> pd.DataFrame:
    rows = [benchmark_result_to_row(result, langs) for result in results]
    df = pd.concat(rows)
    df = df.sort_values(by="Average Score", ascending=False)
    df["Average Rank"] = compute_avg_rank(df)

    # ensure that the average and open source are the first column
    cols = df.columns.tolist()
    first_columns = [
        "Average Score",
        "Average Rank",
    ]
    other_cols = sorted(c for c in cols if c not in first_columns)
    df = df[first_columns + other_cols]
    df = df.drop(columns=["Open Source", "Embedding Size"])

    # convert name to column
    df = df.reset_index()
    df = df.rename(columns={"index": "Model"})
    ranks = [i + 1 for i in range(len(df.index))]
    df.insert(0, "Rank", ranks)  # type: ignore
    return df


def compute_avg_rank(df: pd.DataFrame) -> pd.Series:
    """
    For each model in the dataset, for each task, compute the rank of the model
    and then compute the average rank.
    """
    df = df.drop(columns=["Average Score", "Open Source", "Embedding Size"])
    ranks = df.rank(axis=0, ascending=False)
    avg_ranks: pd.Series = ranks.mean(axis=1)  # type: ignore
    return avg_ranks


MIN_WIDTHS = {
    "Rank": len("Rank"),
    "Model": len("embed-multilingual-v3.0"),
    "Average Score": len("Average"),
    "Average Rank": len("Average"),
}
NO_WRAP = {
    "Model": True,
    "Rank": True,
}


def pretty_print_benchmark(df: pd.DataFrame, highlight: list[str]):
    """Pretty prints the benchmark's results with Rich.
    If you pass a model name in highlight, the model will
    be highlighted and only the rows around it will be showed,
    otherwise the full benchmark is shown and no row is highlighted.
    """
    console = Console()
    table = Table(title="Benchmark Results")
    for column in df.columns:
        justify = "left" if column == "Model" else "right"
        table.add_column(
            column,
            justify=justify,
            overflow="ellipsis",
            min_width=MIN_WIDTHS.get(column, None),
            no_wrap=NO_WRAP.get(column, False),
        )
    if highlight:
        models_to_display = []
        # Add top 3 models
        models_to_display.extend(df["Model"][df["Rank"] <= 3])
        # Add models surrounding the highlighted ones
        for model in highlight:
            model_rank = df[df["Model"] == model]["Rank"].iloc[0]  # type: ignore
            models_to_display.extend(df["Model"][(df["Rank"] - model_rank).abs() < 2])
    else:
        models_to_display = list(df["Model"])
    df = df[df["Model"].isin(models_to_display)]  # type: ignore
    df = df.sort_values("Rank")  # type: ignore
    for _, row in df.iterrows():
        style = "deep_sky_blue1 bold" if (row["Model"] in highlight) else None
        rank = row["Rank"]
        values = []
        for val in row:
            if isinstance(val, float):
                val = f"{val:.2f}"
            if not isinstance(val, str):
                val = str(val)
            values.append(val)
        table.add_row(*values, style=style)
        if (rank == 3) and highlight:
            table.add_section()
    # console.clear()
    console.print(table)



"""
The test specifications for the benchmark.
"""

from datetime import datetime
from pathlib import Path
from typing import Optional, Union

import pytest
import seb

from .dummy_model import create_test_model
from .dummy_task import (
    create_test_encode_task,
    create_test_raise_error_task,
    create_test_task,
)
from .test_tasks import all_tasks_names


@pytest.mark.parametrize(
    ("model_names", "languages", "tasks"),
    [
        (
            ["sentence-transformers/all-MiniLM-L6-v2"],
            None,
            [create_test_task()],
        ),
        (
            [
                "test_model",
                "sentence-transformers/all-MiniLM-L6-v2",
            ],
            None,
            [create_test_task(), create_test_encode_task()],
        ),
    ],
)
def test_run_benchmark(
    model_names: list[str],
    languages: Optional[list[str]],
    tasks: Optional[Union[list[str], list[seb.Task]]],
    tmp_path: Path,
):
    """
    Test that the benchmark runs without errors.
    """
    models = [seb.get_model(model_name) for model_name in model_names]

    if tasks is None:
        tasks = all_tasks_names  # to avoid test tasks that raise errors

    benchmark: seb.Benchmark = seb.Benchmark(
        languages=languages,
        tasks=tasks,
    )
    bm_results: list[seb.BenchmarkResults] = benchmark.evaluate_models(models=models, use_cache=False, cache_dir=tmp_path)

    assert len(bm_results) == len(models)

    n_tasks = len(benchmark.tasks)
    for bm_result in bm_results:
        assert len(bm_result) == n_tasks
        for task_result in bm_result:
            ensure_correct_task_result(task_result)


def ensure_correct_task_result(task_result: Union[seb.TaskResult, seb.TaskError]):
    """
    Ensure that the task result is correct.
    """

    assert isinstance(task_result, seb.TaskResult)
    assert isinstance(task_result.task_name, str)
    assert isinstance(task_result.languages, list)

    main_score = task_result.get_main_score()
    assert isinstance(main_score, float)


@pytest.mark.parametrize(
    ("model_name", "languages", "tasks"),
    [
        (
            "sentence-transformers/all-MiniLM-L6-v2",
            None,
            [create_test_encode_task()],
        ),
    ],
)
def test_cache_dir_is_reused(
    model_name: str,
    languages: Optional[list[str]],
    tasks: Optional[Union[list[str], list[seb.Task]]],
    tmp_path: Path,
):
    """
    Check that the cache dir is reused.
    """
    model = seb.get_model(model_name)
    benchmark: seb.Benchmark = seb.Benchmark(
        languages=languages,
        tasks=tasks,
    )

    before_run = datetime.now()
    bm_result: seb.BenchmarkResults = benchmark.evaluate_model(model, use_cache=False, cache_dir=tmp_path)
    after_run = datetime.now()

    assert len(bm_result) == 1
    task_result_1 = bm_result[0]
    not_used_cache = before_run < task_result_1.time_of_run < after_run
    assert not_used_cache

    bm_result: seb.BenchmarkResults = benchmark.evaluate_model(
        model=model,
        use_cache=True,
        cache_dir=tmp_path,
    )

    assert isinstance(bm_result, seb.BenchmarkResults)

    task_result_2 = bm_result[0]

    used_cache = task_result_1.time_of_run == task_result_2.time_of_run
    assert used_cache
    assert task_result_1 == task_result_2, "The two task results should be equal"


def test_benchmark_skip_on_error_raised(tmp_path: Path):
    """
    Test that the benchmark skips a model if an error is raised.
    """
    task = create_test_raise_error_task()
    model = seb.get_model("test_model")
    benchmark: seb.Benchmark = seb.Benchmark(
        languages=None,
        tasks=[task],
    )

    bm_result: seb.BenchmarkResults = benchmark.evaluate_model(
        model,
        use_cache=False,
        raise_errors=False,
        cache_dir=tmp_path,
    )

    assert len(bm_result) == 1
    task_result = bm_result[0]
    assert task_result.task_name == "test raise error task"
    assert isinstance(task_result, seb.TaskError)

    # test that the benchmark raises an error if raise_errors is True
    with pytest.raises(ValueError):  # noqa: PT011
        benchmark.evaluate_model(model, use_cache=False, raise_errors=True, cache_dir=tmp_path)


@pytest.mark.parametrize("languages", [None, ["nb", "nn"], ["sv", "nb", "nn"], ["sv", "nb", "nn", "da"]])
@pytest.mark.parametrize("tasks", [None, ["DKHate", "ScaLA"]])
def test_benchmark_init(languages: Optional[list[str]], tasks: Optional[list[str]]):
    benchmark = seb.Benchmark(languages=languages, tasks=tasks)

    if languages:
        lang_set = set(languages)
        for task in benchmark.tasks:
            assert any(lang in lang_set for lang in task.languages), "the benchmark should not include tasks with languages not specified"
    if tasks:
        tasks_names = {t.name for t in benchmark.tasks}
        assert tasks_names.issubset(set(tasks)), "The tasks should be the same or less than the tasks specified"



from datetime import datetime
from typing import Any

import seb


class TestTask(seb.Task):
    name = "test task"
    main_score = "a_metric"
    description = "NA"
    reference = "NA"
    version = "NA"
    languages = []  # noqa: RUF012
    domain = []  # noqa: RUF012
    task_type = "Classification"

    def evaluate(self, model: seb.Encoder) -> seb.TaskResult:  # noqa: ARG002
        return seb.TaskResult(
            task_name="test task",
            task_description="NA",
            task_version="NA",
            time_of_run=datetime.now(),
            scores={"nb": {"a_metric": 1.0}},
            main_score="a_metric",
        )

    def get_descriptive_stats(self) -> dict[str, Any]:
        return {}


def create_test_task() -> seb.Task:
    return TestTask()  # type: ignore


def create_test_encode_task() -> seb.Task:
    class TestTaskWithEncode(TestTask):
        name = "test encode task"

        def evaluate(self, model: seb.Encoder) -> seb.TaskResult:
            model.encode(["a test sentence"], task=self)

            return seb.TaskResult(
                task_name="test task",
                task_description="NA",
                task_version="NA",
                time_of_run=datetime.now(),
                scores={"nb": {"a_metric": 1.0}},
                main_score="a_metric",
            )

    return TestTaskWithEncode()


def create_test_raise_error_task() -> seb.Task:
    """
    Note this task is not registered as it will cause errrors in other tests.
    """

    class TestTaskWithError(TestTask):
        name = "test raise error task"

        def evaluate(self, model: seb.Encoder) -> seb.TaskResult:  # noqa ARG002
            raise ValueError("Test raised error. This error should be handled.")

    return TestTaskWithError()



"""
The openai embedding api's evaluated on the SEB benchmark.
"""


import logging
from collections.abc import Sequence
from datetime import date
from functools import partial
from typing import Any

import torch

from seb.interfaces.model import Encoder, LazyLoadEncoder, ModelMeta, SebModel
from seb.interfaces.task import Task
from seb.registries import models

logger = logging.getLogger(__name__)


class OpenaiTextEmbeddingModel(Encoder):
    def __init__(self, api_name: str, input_sentences: int = 64) -> None:
        self.api_name = api_name
        self.input_sentences = input_sentences

    @staticmethod
    def preprocess(sentences: Sequence[str]) -> Sequence[str]:
        return [t.replace("\n", " ") for t in sentences]

    @staticmethod
    def create_sentence_blocks(
        sentences: Sequence[str],
        block_size: int,
    ) -> list[Sequence[str]]:
        sent_blocks: list[Sequence[str]] = []
        for i in range(0, len(sentences), block_size):
            sent_blocks.append(sentences[i : i + block_size])
        return sent_blocks

    def get_embedding_dim(self) -> int:
        v = self.embed(["get emb dim"])
        return v.shape[1]

    def embed(self, sentences: Sequence[str]) -> torch.Tensor:
        import openai  # type: ignore
        from openai.error import InvalidRequestError  # type: ignore

        try:
            emb = openai.Embedding.create(  # type: ignore
                input=sentences,
                model=self.api_name,
            )
        except InvalidRequestError as e:
            if "Please reduce your prompt" in e._message:  # type: ignore
                if len(sentences) == 1:
                    logger.warning("Text is too long for the API, truncating.")
                    max_size_in_str_text = sentences[0][:50_000]
                    half = len(max_size_in_str_text) // 2
                    first_half = max_size_in_str_text[:half]
                    return self.embed([first_half])

                half = len(sentences) // 2
                return torch.cat(
                    [
                        self.embed(sentences[:half]),
                        self.embed(sentences[half:]),
                    ],
                )
            raise e
        data = emb["data"]  # type: ignore
        vectors = [embedding.embedding for embedding in data]
        return torch.tensor(vectors)

    def encode(  # type: ignore
        self,
        sentences: Sequence[str],
        *,
        task: "Task",  # noqa: ARG002
        batch_size: int = 32,  # noqa: ARG002
        **kwargs: Any,  # noqa: ARG002
    ) -> torch.Tensor:
        sentences = self.preprocess(sentences)
        sent_blocks = self.create_sentence_blocks(sentences, self.input_sentences)

        vectors = []

        for sent_block in sent_blocks:
            vectors.append(self.embed(sent_block))

        return torch.cat(vectors)


@models.register("text-embedding-ada-002")
def create_openai_ada_002() -> SebModel:
    api_name = "text-embedding-ada-002"
    meta = ModelMeta(
        name=api_name,
        huggingface_name=None,
        reference="https://openai.com/blog/new-and-improved-embedding-model",
        languages=[],
        open_source=False,
        embedding_size=1536,
        model_type="API",
        release_date=date(2022, 1, 25),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(OpenaiTextEmbeddingModel, api_name=api_name)),
        meta=meta,
    )


@models.register("text-embedding-3-small")
def create_openai_3_small() -> SebModel:
    api_name = "text-embedding-3-small"
    meta = ModelMeta(
        name=api_name,
        huggingface_name=None,
        reference="https://openai.com/blog/new-and-improved-embedding-model",
        languages=[],
        open_source=False,
        embedding_size=1536,
        model_type="API",
        release_date=date(2024, 1, 25),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(OpenaiTextEmbeddingModel, api_name=api_name)),
        meta=meta,
    )


@models.register("text-embedding-3-large")
def create_openai_3_large() -> SebModel:
    api_name = "text-embedding-3-large"
    meta = ModelMeta(
        name=api_name,
        huggingface_name=None,
        reference="https://openai.com/blog/new-and-improved-embedding-model",
        languages=[],
        open_source=False,
        embedding_size=3072,
        model_type="API",
        release_date=date(2024, 1, 25),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(OpenaiTextEmbeddingModel, api_name=api_name)),
        meta=meta,
    )



from pathlib import Path

import pandas as pd
from seb import registries
from tqdm import tqdm


def insert_table(file: Path, table: str) -> None:
    # Read the original Markdown file
    with file.open("r") as f:
        content = f.read()

    # Split the content at the start and end placeholders
    start_section, end_section = (
        content.split("<!--START_TABLE-->")[0],
        content.split("<!--END_TABLE-->")[1],
    )

    # Construct the updated content
    updated_content = start_section + "<!--START_TABLE-->\n" + table + "\n<!--END_TABLE-->" + end_section

    # Write the updated content back to the file
    with file.open("w") as f:
        f.write(updated_content)


def create_table() -> pd.DataFrame:
    # Initialize an empty DataFrame
    columns = [
        "Dataset",
        "Description",
        "Main Score",
        "Languages",
        "Type",
        "Domains",
        "Number of Documents",
        "Mean Length of Documents (characters)",
    ]
    data = []

    tasks = registries.tasks.get_all().items()
    tasks = sorted(tasks, key=lambda x: x[0])

    pbar = tqdm(tasks, desc="Creating Table")
    for name, getter in pbar:
        pbar.set_postfix_str(name)
        task = getter()
        stats = task.get_descriptive_stats()

        # Create a row for each task
        row = [
            f"[{task.name}]({task.reference})",  # Dataset with hyperlink
            task.description,  # Description
            task.main_score.capitalize(),  # Main Score
            ", ".join(task.languages),  # Languages
            task.task_type,  # Type
            ", ".join(task.domain),  # Domains
            stats["num_documents"],  # Number of Documents
            f'{stats["mean_document_length"]:.2f} (std: {stats["std_document_length"]:.2f})',  # Mean Length of Documents
        ]
        data.append(row)

    # Convert the data to a DataFrame
    df = pd.DataFrame(data, columns=columns)

    return df


def main():
    path = Path(__file__).parent / "datasets.md"

    df = create_table()
    insert_table(path, df.to_markdown(index=False))


if __name__ == "__main__":
    main()



from typing import Literal, Protocol, TypedDict, runtime_checkable

import nummy as np

from seb.interfaces.language import Language

from ..result_dataclasses import TaskResult
from .model import Encoder

Domain = Literal[
    "social",
    "poetry",
    "wiki",
    "fiction",
    "non-fiction",
    "web",
    "legal",
    "news",
    "academic",
    "spoken",
    "reviews",
    "blog",
    "medical",
    "government",
    "bible",
]

TaskType = Literal["Classification", "Retrieval", "STS", "BitextMining", "Clustering", "Speed"]


class DescriptiveDatasetStats(TypedDict):
    mean_document_length: float
    std_document_length: float
    num_documents: int


@runtime_checkable
class Task(Protocol):
    """
    A task is a specific evaluation task for a sentence embedding model.

    Attributes:
        name: The name of the task.
        main_score: The main score of the task.
        reference: A reference to the task.
        version: The version of the task.
        languages: The languages of the task.
        domain: The domains of the task. Should be one of the categories listed on https://universaldependencies.org
        description: A description of the task.
    """

    name: str
    main_score: str
    reference: str
    version: str
    languages: list[Language]
    domain: list[Domain]
    task_type: TaskType
    description: str

    def evaluate(self, model: Encoder) -> TaskResult:
        """
        Evaluates a Sentence Embedding Model on the task.

        Args:
            model: A model with the encode method implemented.

        Returns:
            A TaskResult object.
        """
        ...

    def get_documents(self) -> list[str]:
        """
        Get the documents for the task.

        Returns:
            A list of strings.
        """
        ...

    def get_descriptive_stats(self) -> DescriptiveDatasetStats:
        texts = self.get_documents()
        document_lengths = np.array([len(text) for text in texts])

        mean = float(np.mean(document_lengths))
        std = float(np.std(document_lengths))
        return DescriptiveDatasetStats(
            mean_document_length=mean,
            std_document_length=std,
            num_documents=len(document_lengths),
        )

    def name_to_path(self) -> str:
        """
        Convert a name to a path.
        """
        name = self.name.replace("/", "__").replace(" ", "_")
        return name



from datetime import datetime
from pathlib import Path

import pytest
import seb
from seb.result_dataclasses import TaskResult

all_tasks = seb.get_all_tasks()
all_tasks_names = [task.name for task in all_tasks if not task.name.startswith("test ")]  # ignore tasks intended for testing


@pytest.fixture()
def task_result() -> seb.TaskResult:
    task_result = seb.TaskResult(
        task_name="test",
        scores={"nb": {"test_measure": 0.4}, "da": {"test_measure": 0.2}},
        main_score="test_measure",
        time_of_run=datetime.now(),
        task_version="0.0.1",
        task_description="Just a test",
    )
    return task_result


def test_all_tasks_are_unique():
    all_tasks_names = [task.name for task in all_tasks]
    assert len(all_tasks_names) == len(set(all_tasks_names))


def test_task_name_is_registried_under_same_name():
    for regristry_name, task_create_fn in seb.tasks.get_all().items():
        task = task_create_fn()
        assert task.name == regristry_name


def test_read_write_tasks(task_result: TaskResult):
    tmp_path = Path("tests/tmpfiles/task_result.json")
    tmp_path.parent.mkdir(parents=True, exist_ok=True)

    task_result.to_disk(tmp_path)
    # check that the file exists
    assert tmp_path.exists()
    task_result_from_disk = seb.TaskResult.from_disk(tmp_path)

    assert task_result_from_disk == task_result


def test_task_result_main_score(task_result: seb.TaskResult):
    assert task_result.get_main_score(["da"]) == 0.2
    assert task_result.get_main_score(["nb"]) == 0.4
    assert task_result.get_main_score() - task_result.get_main_score(["da", "nb"]) < 0.0001
    assert task_result.get_main_score(["da", "nb"]) - 0.3 < 0.0001


@pytest.mark.skip(
    reason="This test downloads all datasets. It takes a long time to test and often fails due to errors on HF's side.",
)
@pytest.mark.parametrize("task_name", all_tasks_names)
@pytest.mark.parametrize("model_name", ["sentence-transformers/all-MiniLM-L6-v2"])
def test_all_tasks(task_name: str, model_name: str):
    task: seb.Task = seb.get_task(task_name)
    model: seb.EmbeddingModel = seb.get_model(model_name)

    assert isinstance(task, seb.Task)
    assert isinstance(model, seb.EmbeddingModel)
    assert isinstance(model.model, seb.Encoder)

    task_result = task.evaluate(model)
    assert isinstance(task_result, seb.TaskResult)
    assert isinstance(task_result.get_main_score(), float)



"""
The dataset is quite similar to the SNL dataset (both are wikistyle), however NorQuad actually uses questions, while the other is just headline,
article pairs.
"""

from typing import Any

import datasets
from mteb.abstasks import AbsTaskRetrieval


class NorQuadRetrieval(AbsTaskRetrieval):
    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "NorQuadRetrieval",
            "hf_hub_name": "ScandEval/norquad-mini",
            "description": "Human-created question for Norwegian wikipedia passages.",
            "reference": "https://aclanthology.org/2023.nodalida-1.17/",
            "type": "Retrieval",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["nb"],
            "main_score": "ndcg_at_10",
            "revision": "a47881440ce4b18ef61a99be66dc4badbf5aac6e",
        }

    def load_data(self, **kwargs: dict):  # noqa: ARG002
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset: datasets.DatasetDict = datasets.load_dataset(
            self.description["hf_hub_name"],
            revision=self.description.get("revision"),
        )  # type: ignore

        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """
        and transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}
        text2id = {}

        for split in self.dataset:
            ds: datasets.Dataset = self.dataset[split]  # type: ignore
            ds = ds.shuffle(seed=42)
            max_samples = min(1024, len(ds))
            ds = ds.select(range(max_samples))  # limit the dataset size to make sure the task does not take too long to run
            self.queries[split] = {}
            self.relevant_docs[split] = {}
            self.corpus[split] = {}

            question = ds["question"]
            context = ds["context"]
            answer = [a["text"][0] for a in ds["answers"]]

            n = 0
            for q, cont, ans in zip(question, context, answer):
                self.queries[split][str(n)] = q
                q_n = n
                n += 1
                if cont not in text2id:
                    text2id[cont] = n
                    self.corpus[split][str(n)] = {"title": "", "text": cont}
                    n += 1
                if ans not in text2id:
                    text2id[ans] = n
                    self.corpus[split][str(n)] = {"title": "", "text": ans}
                    n += 1

                self.relevant_docs[split][str(q_n)] = {str(text2id[ans]): 1, str(text2id[cont]): 1}  # only two correct matches



from typing import Any

import datasets
from mteb.abstasks import AbsTaskRetrieval


class SweFaqRetrieval(AbsTaskRetrieval):
    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "swefaq",
            "hf_hub_name": "AI-Sweden/SuperLim",
            "description": "A Swedish QA dataset derived from FAQ",
            "reference": "https://spraakbanken.gu.se/en/resources/superlim",
            "type": "Retrieval",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["sv"],
            "main_score": "ndcg_at_10",
            "revision": "7ebf0b4caa7b2ae39698a889de782c09e6f5ee56",
        }

    def load_data(self, **kwargs: dict):  # noqa: ARG002
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset: datasets.DatasetDict = datasets.load_dataset(
            self.description["hf_hub_name"],
            "swefaq",  # chose the relevant subset
            revision=self.description.get("revision"),
        )  # type: ignore

        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """
        and transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}
        text2id = {}

        for split in self.dataset:
            ds: datasets.Dataset = self.dataset[split]  # type: ignore
            self.queries[split] = {}
            self.relevant_docs[split] = {}
            self.corpus[split] = {}

            questions = ds["question"]
            ca_answers = ds["candidate_answer"]
            co_answers = ds["correct_answer"]

            n = 0
            for q, ca, co in zip(questions, ca_answers, co_answers):
                self.queries[split][str(n)] = q
                q_n = n
                n += 1
                if ca not in text2id:
                    text2id[ca] = n
                    self.corpus[split][str(n)] = {"title": "", "text": ca}
                    n += 1
                if co not in text2id:
                    text2id[co] = n
                    self.corpus[split][str(n)] = {"title": "", "text": co}
                    n += 1
                cor_n = text2id[co]

                self.relevant_docs[split][str(q_n)] = {
                    str(cor_n): 1,
                }  # only one correct match



from .dan_fever import DanFever
from .swefaq_retrieval import SweFaqRetrieval
from .swedn_retrieval import SwednRetrieval
from .swefaq_retrieval import SweFaqRetrieval
from .snl_retrieval import SNLRetrieval
from .t2nord_retrieval import TV2Nordretrieval
from .norquad import NorQuadRetrieval
from .twitterhjerne import TwitterHjerneRetrieval


from .benchmark import Benchmark, run_task
from .full_benchmark import run_benchmark, run_speed_benchmark
from .registries import (
    get_all_models,
    get_all_tasks,
    get_model,
    get_task,
    models,
    tasks,
)

from .interfaces.model import Encoder, LazyLoadEncoder, ModelMeta, SebModel
from .interfaces.task import Task
from .registered_models import *  # import all SEB models
from .registered_tasks import *  # import all SEB tasks
from .result_dataclasses import BenchmarkResults, TaskError, TaskResult



from .cohere_models import *
from .e5_mistral import *
from .e5_models import *
from .fairseq_models import *
from .fasttext import *
from .hf_models import *
from .openai_models import *
from .translate_e5_models import *



from dataclasses import dataclass
from pathlib import Path
from typing import Union

import numpy as np
import pytest

import seb
from seb.cli import cli, run_benchmark_cli

test_dir = Path(__file__).parent


@dataclass
class BenchmarkCliTestInput:
    model: str
    score: float
    tasks: Union[list[str], None] = None
    languages: Union[list[str], None] = None
    code_path: Union[Path, None] = None
    ignore_cache: bool = False

    def to_command(self, output_path: Path) -> list[str]:
        cmd = ["", "run", "-m", self.model, "--output-path", f"{output_path}"]
        cmd += ["--languages", f"{','.join(self.languages)}"] if self.languages else []
        cmd += ["-t", f"{','.join(self.tasks)}"] if self.tasks else []
        cmd += ["--code", f"{self.code_path}"] if self.code_path else []
        return cmd


cli_command_parametrize = pytest.mark.parametrize(
    "inputs",
    [
        BenchmarkCliTestInput("sentence-transformers/all-MiniLM-L6-v2", 0.550, tasks=["DKHate"]),
        BenchmarkCliTestInput("sentence-transformers/all-MiniLM-L6-v2", 0.525, tasks=["DKHate", "ScaLA"]),
        BenchmarkCliTestInput(
            "sentence-transformers/all-MiniLM-L6-v2",
            0.50,
            tasks=["DKHate", "ScaLA"],
            languages=["sv", "nn", "nb"],
        ),
        BenchmarkCliTestInput(
            "test_model",
            np.nan,
            code_path=(test_dir / "benchmark_cli_code_inject.py"),
            tasks=["test-encode-task"],
            ignore_cache=True,
        ),
    ],
)


def is_approximately_equal(a: float, b: float) -> bool:
    """nan safe equality check"""
    if np.isnan(a) and np.isnan(b):
        return True
    return abs(a - b) < 0.001


def load_results(path: Path) -> list[seb.BenchmarkResults]:
    return [seb.BenchmarkResults.from_disk(d) for d in path.iterdir() if d.is_dir()]


@cli_command_parametrize
def test_run_benchmark_cli(inputs: BenchmarkCliTestInput, tmp_path: Path):
    run_benchmark_cli(
        models=[inputs.model],
        languages=inputs.languages,
        tasks=inputs.tasks,
        output_path=tmp_path,
        code_path=inputs.code_path,
        ignore_cache=inputs.ignore_cache,
    )
    res = load_results(tmp_path)
    assert len(res) == 1
    bench_res = res[0]
    bench_res.task_results = [tr for tr in bench_res.task_results if tr.task_name != "test-encode-task"]
    assert is_approximately_equal(bench_res.get_main_score(), inputs.score)


@cli_command_parametrize
def test_run_cli(inputs: BenchmarkCliTestInput, tmp_path: Path):
    cli.run(inputs.to_command(tmp_path))
    res = load_results(tmp_path)

    assert len(res) == 1
    bench_res = res[0]
    bench_res.task_results = [tr for tr in bench_res.task_results if tr.task_name != "test-encode-task"]
    assert is_approximately_equal(bench_res.get_main_score(), inputs.score)



import catalogue

from .interfaces.model import SebModel
from .interfaces.task import Task

models = catalogue.create("seb", "models")
tasks = catalogue.create("seb", "tasks")


def get_model(name: str) -> SebModel:
    """
    Fetches a model by name.

    Args:
        name: The name of the model.

    Returns:
        A model including metadata.
    """
    return models.get(name)()


def get_task(name: str) -> Task:
    """
    Fetches a task by name.

    Args:
        name: The name of the task.

    Returns:
        A task.
    """
    return tasks.get(name)()


def get_all_tasks() -> list[Task]:
    """
    Returns all tasks implemented in SEB.

    Returns:
        A list of all tasks in SEB.
    """
    return [get_task(task_name) for task_name in tasks.get_all()]


def get_all_models() -> list[SebModel]:
    """
    Get all the models implemented in SEB.

    Returns:
        A list of all models in SEB.
    """
    return [get_model(model_name) for model_name in models.get_all()]



"""
This section contains big tests which are too heavy to run as part of the CI, but are nice to have
"""

import pytest
import seb

from .dummy_task import create_test_encode_task

all_models = seb.get_all_models()
openai_models = []


@pytest.mark.skip(
    reason="This test loads in all models. It is too heavy to have running as a CI",
)
@pytest.mark.parametrize("model", all_models)
@pytest.mark.parametrize("task", [create_test_encode_task()])
def test_model(model: seb.SebModel, task: seb.Task):
    """
    Test if the models encodes as expected
    """
    task.evaluate(model.encoder)


@pytest.mark.skip(
    reason="This test loads in all models. It is too heavy to have running as a CI",
)
@pytest.mark.parametrize("model", all_models)
def test_embedding_match_what_is_stated(model: seb.SebModel):
    """
    This test checks if the embedding size matches what is stated in the ModelMeta
    """
    output = model.encoder.encode(["test"])
    output_embedding_size = output.shape[1]
    assert output_embedding_size == model.meta.embedding_size


@pytest.mark.skip(
    reason="This test applied the openai embedding models. It is too expensive to have running as a CI",
)
@pytest.mark.parametrize("model", [seb.get_model("text-embedding-ada-002")])
@pytest.mark.parametrize("task", [create_test_encode_task()])
def test_openai_model(model: seb.SebModel, task: seb.Task):
    """
    Test if the models encodes as expected
    """
    task.evaluate(model.encoder)



from seb.interfaces.mteb_task import MTEBTask
from seb.interfaces.task import Task
from seb.registries import tasks


@tasks.register("SweReC")
def create_swerec() -> Task:
    from mteb import SweRecClassification

    task = MTEBTask(SweRecClassification())
    task.name = "SweReC"
    task.domain = ["reviews"]
    return task


@tasks.register("DaLAJ")
def create_dalaj() -> Task:
    from mteb import DalajClassification

    task = MTEBTask(DalajClassification())
    task.name = "DaLAJ"
    task.domain = ["fiction", "non-fiction"]
    return task


@tasks.register("SweFAQ")
def create_swefaq() -> Task:
    from seb.mteb_tasks import SweFaqRetrieval

    task = MTEBTask(SweFaqRetrieval())
    task.name = "SweFAQ"
    task.version = "0.0.1"
    task.domain = ["non-fiction", "web"]
    task._text_columns = ["question", "candidate_answer", "correct_answer"]
    return task


@tasks.register("SwednRetrieval")
def create_swedn_retrieval() -> Task:
    from seb.mteb_tasks import SwednRetrieval

    task = MTEBTask(SwednRetrieval())
    task.name = "SwednRetrieval"
    task.version = "0.0.1"
    task.domain = ["non-fiction", "news"]
    return task


@tasks.register("SwednClustering")
def create_swedn_clustering() -> Task:
    from seb.mteb_tasks import SwednClustering

    task = MTEBTask(SwednClustering())
    task.name = "SwednClustering"
    task.version = "0.0.1"
    task.domain = ["non-fiction", "news"]
    task._text_columns = ["sentences"]
    return task



from datetime import datetime
from functools import partial

import numpy as np
from datasets import DatasetDict, concatenate_datasets

from seb.interfaces.model import Encoder
from seb.interfaces.mteb_task import MTEBTask
from seb.interfaces.task import DescriptiveDatasetStats, Task
from seb.registries import tasks
from seb.result_dataclasses import TaskResult


@tasks.register("Massive Intent")
def create_massive_intent() -> Task:
    from mteb import MassiveIntentClassification

    task = MTEBTask(MassiveIntentClassification())
    task.name = "Massive Intent"
    task.languages = ["da", "nb", "sv"]
    task.mteb_task.langs = ["da", "nb", "sv"]  # type: ignore
    task.domain = ["spoken"]
    return task


@tasks.register("Massive Scenario")
def create_massive_scenario() -> Task:
    from mteb import MassiveScenarioClassification

    task = MTEBTask(MassiveScenarioClassification())
    task.name = "Massive Scenario"
    task.languages = ["da", "nb", "sv"]
    task.mteb_task.langs = ["da", "nb", "sv"]  # type: ignore
    task.domain = ["spoken"]
    return task


@tasks.register("ScaLA")
def create_scala() -> Task:
    from mteb import ScalaDaClassification, ScalaNbClassification, ScalaNnClassification, ScalaSvClassification, __version__

    class ScalaTask(Task):
        def __init__(self) -> None:
            self.mteb_tasks = {
                "da": ScalaDaClassification(),
                "nb": ScalaNbClassification(),
                "sv": ScalaSvClassification(),
                "nn": ScalaNnClassification(),
            }
            self.main_score = "accuracy"
            self.name = "ScaLA"
            self.description = "A linguistic acceptability task for Danish, Norwegian Bokmål Norwegian Nynorsk and Swedish."
            self.version = __version__
            self.reference = "https://aclanthology.org/2023.nodalida-1.20/"
            self.languages = ["da", "nb", "sv", "nn"]
            self.domain = ["fiction", "news", "non-fiction", "spoken", "blog"]
            self._text_columns = ["text"]
            self.task_type = "Classification"

        def load_data(self) -> DatasetDict:
            ds = {}
            for lang, mteb_task in self.mteb_tasks.items():  # noqa: B007
                mteb_task.load_data()
                for split in mteb_task.dataset:
                    if split not in ds:
                        ds[split] = mteb_task.dataset[split]
                    else:
                        ds[split] = concatenate_datasets(
                            [ds[split], mteb_task.dataset[split]],
                        )

            return DatasetDict(ds)

        def get_documents(self) -> list[str]:
            ds = self.load_data()
            texts = []
            splits = self.get_splits()
            assert len(splits) >= 1, "No splits found in MTEB task."

            for split in splits:
                for text_column in self._text_columns:
                    texts += ds[split][text_column]
            return texts

        def get_descriptive_stats(self) -> DescriptiveDatasetStats:
            texts = self.get_documents()
            document_lengths = np.array([len(text) for text in texts])

            mean = np.mean(document_lengths)
            std = np.std(document_lengths)
            return DescriptiveDatasetStats(
                mean_document_length=float(mean),
                std_document_length=float(std),
                num_documents=len(document_lengths),
            )

        def evaluate(self, model: Encoder) -> TaskResult:
            scores = {}
            # Infusing task into encode()
            original_encode = model.encode
            try:
                model.encode = partial(model.encode, task=self)
                for lang, mteb_task in self.mteb_tasks.items():
                    mteb_task.load_data()
                    score = mteb_task.evaluate(model)
                    scores[lang] = score
                model.encode = original_encode
            except Exception as e:
                raise e
            finally:
                # Resetting encode to original
                model.encode = original_encode

            return TaskResult(
                task_name=self.name,
                task_version=self.version,
                time_of_run=datetime.now(),
                scores=scores,
                task_description=self.description,
                main_score=self.main_score,
            )

        def get_splits(self) -> list[str]:
            splits = []
            for lang, mteb_task in self.mteb_tasks.items():  # noqa: B007
                splits += mteb_task.description["eval_splits"]
            return list(set(splits))

    return ScalaTask()


@tasks.register("Language Identification")
def create_language_identification() -> Task:
    from mteb import NordicLangClassification

    task = MTEBTask(NordicLangClassification())
    task.name = "Language Identification"
    task.domain = ["wiki"]

    return task



"""
Script for running the benchmark and pushing the results to Datawrapper.

Example:

    python update_benchmark_tables.py --data-wrapper-api-token <token>
"""
import argparse
from collections.abc import Sequence
from typing import Optional

import numpy as np
import pandas as pd
import seb
from datawrapper import Datawrapper
from seb.full_benchmark import BENCHMARKS
from seb.registered_tasks.speed import CPUSpeedTask

subset_to_chart_id = {
    "Mainland Scandinavian": "7Nwjx",
    "Danish": "us1YK",
    "Norwegian": "pV87q",
    "Swedish": "aL23t",
    "Domain": "F00q5",
    "Task Type": "4jkip",
    "Speed x Performance": "oXdUJ",
}

datawrapper_lang_codes = {
    "da": "dk",
    "nb": "no",
    "nn": "no",
    "sv": "se",
    "en": "us",
}


def get_main_score(task: seb.TaskResult, langs: list[str]) -> float:
    _langs = set(langs) & set(task.languages)
    return task.get_main_score(_langs) * 100


def get_flag(languages: Sequence[str]) -> str:
    if languages:
        flags = []
        for l in languages:
            if l in datawrapper_lang_codes:
                flags.append(datawrapper_lang_codes[l])
        flags = list(set(flags))  # remove duplicates (e.g. nb and nn)
        return " ".join([f":{f}:" for f in flags])
    return "🌐"


def open_source_to_string(open_source: bool) -> str:
    return "✓" if open_source else "✗"


def create_mdl_name_w_reference(mdl: seb.ModelMeta) -> str:
    reference = mdl.reference
    name: str = mdl.name

    mdl_name = f"[{name}]({reference})" if reference else name
    lang_flag = get_flag(mdl.languages)
    mdl_name = f"{mdl_name} {lang_flag}"

    return mdl_name


def get_speed_results(model_meta: seb.ModelMeta) -> Optional[float]:
    hf_name = model_meta.huggingface_name
    model = seb.get_model(hf_name) if hf_name else seb.get_model(model_meta.name)
    TOKENS_IN_UGLY_DUCKLING = 3591

    speed_task = CPUSpeedTask()
    speed_result = seb.run_task(speed_task, model, raise_errors=False, use_cache=True, run_model=False)
    if isinstance(speed_result, seb.TaskResult):
        speed_in_seconds = speed_result.get_main_score()
        word_per_seconds = TOKENS_IN_UGLY_DUCKLING / speed_in_seconds
        return word_per_seconds
    return None


def benchmark_result_to_row(
    result: seb.BenchmarkResults,
    langs: list[str],
) -> pd.DataFrame:
    mdl_name_w_link = create_mdl_name_w_reference(result.meta)
    # sort by task name
    task_results = result.task_results
    sorted_tasks = sorted(task_results, key=lambda t: t.task_name)
    task_names = [t.task_name for t in sorted_tasks]
    scores = [get_main_score(t, langs) for t in sorted_tasks]  # type: ignore

    df = pd.DataFrame([scores], columns=task_names, index=[mdl_name_w_link])
    df["Model name"] = result.meta.name
    df["Average Score"] = result.get_main_score() * 100
    df["Open Source"] = open_source_to_string(result.meta.open_source)
    df["Embedding Size"] = result.meta.embedding_size
    df["WPS (CPU)"] = get_speed_results(result.meta)
    return df


def create_n_datasets_row_for_domains() -> pd.DataFrame:
    tasks: list[seb.Task] = seb.get_all_tasks()
    domains = sorted({d for t in tasks for d in t.domain})
    domain2tasks = {d: [t.name for t in tasks if d in t.domain] for d in domains}
    scores = []
    domain_names = []
    n_datasets = []
    for d, ts in domain2tasks.items():
        domain_names.append(d.capitalize())
        n_datasets.append(len(ts))
    return pd.DataFrame([n_datasets], columns=domain_names, index=["N. Datasets"])


def create_n_datasets_row_for_task_types() -> pd.DataFrame:
    tasks: list[seb.Task] = seb.get_all_tasks()
    task_type = sorted({t.task_type for t in tasks})
    tasktype2tasks = {tt: [t.name for t in tasks if tt == t.task_type] for tt in task_type}
    scores = []
    task_type_names = []
    n_datasets = []
    for t, ts in tasktype2tasks.items():
        task_type_names.append(t.capitalize())
        n_datasets.append(len(ts))
    return pd.DataFrame([n_datasets], columns=task_type_names, index=["N. Datasets"])


def benchmark_result_to_domain_row(
    result: seb.BenchmarkResults,
    langs: list[str],
) -> pd.DataFrame:
    tasks: list[seb.Task] = seb.get_all_tasks()
    domains = sorted({d for t in tasks for d in t.domain})
    domain2tasks = {d: [t.name for t in tasks if d in t.domain] for d in domains}

    scores = []
    domain_names = []
    n_datasets = []
    for d, ts in domain2tasks.items():
        task_results = [r for r in result.task_results if r.task_name in ts]
        _scores = np.array([get_main_score(t, langs) for t in task_results])  # type: ignore
        score = np.mean(_scores)
        scores.append(score)
        domain_names.append(d.capitalize())
        n_datasets.append(len(ts))

    mdl_name = create_mdl_name_w_reference(result.meta)
    df = pd.DataFrame([scores], columns=domain_names, index=[mdl_name])
    df["Average Score"] = result.get_main_score() * 100
    df["Open Source"] = open_source_to_string(result.meta.open_source)
    df["Embedding Size"] = result.meta.embedding_size
    df["WPS (CPU)"] = get_speed_results(result.meta)
    return df


def benchmark_result_to_task_type_row(
    result: seb.BenchmarkResults,
    langs: list[str],
) -> pd.DataFrame:
    tasks: list[seb.Task] = seb.get_all_tasks()
    task_type = sorted({t.task_type for t in tasks})
    tasktype2tasks = {tt: [t.name for t in tasks if tt == t.task_type] for tt in task_type}

    scores = []
    task_type_names = []
    n_datasets = []
    for t, ts in tasktype2tasks.items():
        task_results = [r for r in result.task_results if r.task_name in ts]
        _scores = np.array([get_main_score(t, langs) for t in task_results])  # type: ignore
        score = np.mean(_scores)
        scores.append(score)
        task_type_names.append(t.capitalize())
        n_datasets.append(len(ts))

    mdl_name = create_mdl_name_w_reference(result.meta)
    df = pd.DataFrame([scores], columns=task_type_names, index=[mdl_name])
    df["Average Score"] = result.get_main_score() * 100
    df["Open Source"] = open_source_to_string(result.meta.open_source)
    df["Embedding Size"] = result.meta.embedding_size
    df["WPS (CPU)"] = get_speed_results(result.meta)
    return df


def convert_to_table(
    results: list[seb.BenchmarkResults],
    langs: list[str],
) -> pd.DataFrame:
    rows = [benchmark_result_to_row(result, langs) for result in results]
    df = pd.concat(rows)
    df = df.sort_values(by="Average Score", ascending=False)
    df["Average Rank"] = compute_avg_rank(df)

    # ensure that the average and open source are the first column
    cols = df.columns.tolist()
    first_columns = ["Average Score", "Average Rank", "Open Source", "Embedding Size", "WPS (CPU)"]
    other_cols = sorted(c for c in cols if c not in first_columns)
    df = df[first_columns + other_cols]

    # convert name to column
    df = df.reset_index()
    df = df.rename(columns={"index": "Model"})
    df = df.sort_values(by="Model", ascending=True)

    return df


def push_to_datawrapper(df: pd.DataFrame, chart_id: str, token: str):
    dw = Datawrapper(access_token=token)
    assert dw.account_info(), "Could not connect to Datawrapper"
    resp = dw.add_data(chart_id, data=df)
    assert 200 <= resp.status_code < 300, "Could not add data to Datawrapper"
    iframe_html = dw.publish_chart(chart_id)
    assert iframe_html, "Could not publish chart"


def compute_avg_rank(df: pd.DataFrame) -> pd.Series:
    """
    For each model in the dataset, for each task, compute the rank of the model and then compute the average rank.
    """
    df = df.drop(columns=["Average Score", "Open Source", "Embedding Size"])

    ranks = df.rank(axis=0, ascending=False, na_option="bottom")
    avg_ranks = ranks.mean(axis=1)
    return avg_ranks


def create_domain_table(
    results: list[seb.BenchmarkResults],
    langs: list[str],
) -> pd.DataFrame:
    rows = [benchmark_result_to_domain_row(result, langs) for result in results]
    df = pd.concat(rows)
    df = pd.concat([df, create_n_datasets_row_for_domains()])
    df = df.sort_values(by="Average Score", ascending=False)
    cols = df.columns.tolist()
    first_columns = ["Average Score", "Open Source", "Embedding Size", "WPS (CPU)"]
    other_cols = sorted(c for c in cols if c not in first_columns)
    df = df[first_columns + other_cols]

    # convert name to column
    df = df.reset_index()
    df = df.rename(columns={"index": "Model"})
    df = df.sort_values(by="Model", ascending=True)
    return df


def create_task_type_table(
    results: list[seb.BenchmarkResults],
    langs: list[str],
) -> pd.DataFrame:
    rows = [benchmark_result_to_task_type_row(result, langs) for result in results]
    df = pd.concat(rows)
    df = pd.concat([df, create_n_datasets_row_for_task_types()])
    df = df.sort_values(by="Average Score", ascending=False)
    cols = df.columns.tolist()
    first_columns = ["Average Score", "Open Source", "Embedding Size", "WPS (CPU)"]
    other_cols = sorted(c for c in cols if c not in first_columns)
    df = df[first_columns + other_cols]

    # convert name to column
    df = df.reset_index()
    df = df.rename(columns={"index": "Model"})
    df = df.sort_values(by="Model", ascending=True)
    return df


def main(data_wrapper_api_token: str):
    results = seb.run_benchmark(use_cache=True, run_models=False, raise_errors=False)

    for subset, result in results.items():
        langs = BENCHMARKS[subset]

        raw_table = convert_to_table(result, langs)
        table = raw_table.drop(columns=["Model name"])
        chart_id = subset_to_chart_id[subset]
        push_to_datawrapper(table, chart_id, data_wrapper_api_token)

        if subset == "Mainland Scandinavian":
            # Update the chart for speed x performance
            chart_id = subset_to_chart_id["Speed x Performance"]
            _table = raw_table.drop(columns=["Model"]).rename(columns={"Model name": "Model"})
            push_to_datawrapper(_table, chart_id, data_wrapper_api_token)

            # also create the summary charts for task types and domains
            table = create_domain_table(result, langs)
            chart_id = subset_to_chart_id["Domain"]
            push_to_datawrapper(table, chart_id, data_wrapper_api_token)

            table = create_task_type_table(result, langs)
            chart_id = subset_to_chart_id["Task Type"]
            push_to_datawrapper(table, chart_id, data_wrapper_api_token)


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--data-wrapper-api-token",
        type=str,
        required=True,
        help="Datawrapper API token",
    )

    args = parser.parse_args()
    main(args.data_wrapper_api_token)



import numpy as np
import seb
from seb.registries import models


@models.register("test_model")
def create_test_model() -> seb.SebModel:
    class TestEncoder(seb.Encoder):
        def encode(
            self,
            sentences: list[str],
            batch_size: int,  # noqa: ARG002
            **kwargs: dict,  # noqa: ARG002
        ) -> np.ndarray:
            # create random array of 100, pr text
            return np.array([np.random.rand(100) for _ in sentences])

    def load_test_model() -> TestEncoder:
        return TestEncoder()

    return seb.SebModel(
        meta=seb.ModelMeta(name="test_model", embedding_size=100),
        encoder=seb.LazyLoadEncoder(load_test_model),
    )



from typing import Any

import datasets
from mteb.abstasks import AbsTaskRetrieval


class DanFever(AbsTaskRetrieval):
    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "DanFEVER",
            "hf_hub_name": "strombergnlp/danfever",
            "description": "A Danish dataset intended for misinformation research. It follows the same format as the English FEVER dataset.",
            "reference": "https://aclanthology.org/2021.nodalida-main.47/",
            "type": "Retrieval",
            "category": "p2p",
            "eval_splits": ["train"],
            "eval_langs": ["da"],
            "main_score": "ndcg_at_10",
            "revision": "5d01e3f6a661d48e127ab5d7e3aaa0dc8331438a",
        }

    def load_data(self, **kwargs: dict):  # noqa: ARG002
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset: datasets.DatasetDict = datasets.load_dataset(
            self.description["hf_hub_name"],
            revision=self.description.get("revision"),
        )  # type: ignore

        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """
        and transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document data like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}
        text2id = {}

        for split in self.dataset:
            self.corpus[split] = {}
            self.relevant_docs[split] = {}
            self.queries[split] = {}

            ds = self.dataset[split]
            claims = ds["claim"]
            evidences = ds["evidence_extract"]
            labels = ds["label"]
            class_labels = ds.features["label"].names

            for claim, evidence, label_id in zip(claims, evidences, labels):
                claim_is_supported = class_labels[label_id] == "Supported"

                sim = 1 if claim_is_supported else 0  # negative for refutes claims - is that what we want?

                if claim not in text2id:
                    text2id[claim] = str(len(text2id))
                if evidence not in text2id:
                    text2id[evidence] = len(text2id)

                claim_id = str(text2id[claim])
                evidence_id = str(text2id[evidence])

                self.queries[split][claim_id] = claim
                self.corpus[split][evidence_id] = {"title": "", "text": evidence}

                if claim_id not in self.relevant_docs[split]:
                    self.relevant_docs[split][claim_id] = {}

                self.relevant_docs[split][claim_id][evidence_id] = sim



from datetime import datetime

import pytest
import seb
from seb.cli import run_benchmark_cli
from seb.registries import tasks


@tasks.register("test-encode-task")
def create_test_encode_task() -> seb.Task:
    class DummyTask(seb.Task):
        name = "test-encode-task"
        main_score = "a_metric"
        description = "NA"
        reference = "NA"
        version = "NA"
        languages = []  # noqa: RUF012
        domain = []  # noqa: RUF012
        task_type = "Classification"

        def evaluate(self, model: seb.Encoder) -> seb.TaskResult:
            model.encode(["a test sentence"], task=self)

            return seb.TaskResult(
                task_name="test-encode-task",
                task_description="NA",
                task_version="NA",
                time_of_run=datetime.now(),
                scores={"en": {"a_metric": 1.0}},
                main_score="a_metric",
            )

        def get_descriptive_stats(self) -> dict:
            return {}

    return DummyTask()


@pytest.mark.parametrize(
    "model",
    [
        "intfloat/e5-small",
        "translate-e5-small",
    ],
)
def test_integration_dummy(model: str):
    """Runs all sorts of models on a dummy task to see if they can run without breaking.
    Cache is ignored so that the models are actually run.
    """
    tasks = ["test-encode-task"]
    run_benchmark_cli(models=[model], tasks=tasks, ignore_cache=True)


@pytest.mark.parametrize(
    "model",
    [
        "sentence-transformers/all-MiniLM-L6-v2",
    ],
)
def test_integration_lcc(model: str):
    """Runs model(s) on LCC to see if everything works in order with tasks included."""
    tasks = ["LCC"]
    run_benchmark_cli(models=[model], tasks=tasks, ignore_cache=True)



from collections.abc import Iterable, Sequence
from datetime import date
from itertools import islice
from typing import Any, Optional, TypeVar

import torch
import torch.nn.functional as F
from numpy.typing import ArrayLike
from torch import Tensor
from transformers import AutoModel, AutoTokenizer, BatchEncoding

from seb.interfaces.model import Encoder, LazyLoadEncoder, ModelMeta, SebModel
from seb.interfaces.task import Task
from seb.registries import models

T = TypeVar("T")


def batched(iterable: Iterable[T], n: int) -> Iterable[tuple[T, ...]]:
    # batched('ABCDEFG', 3) --> ABC DEF G
    if n < 1:
        raise ValueError("n must be at least one")
    it = iter(iterable)
    while batch := tuple(islice(it, n)):
        yield batch


def task_to_instruction(task: Task) -> str:
    if task.task_type in ["STS"]:
        return "Retrieve semantically similar text"
    if task.task_type in ["Summarization"]:
        return "Given a news summary, retrieve other semantically similar summaries"
    if task.task_type in ["BitextMining"]:
        task_name_to_instruct: dict[str, str] = {
            "Bornholm Parallel": "Retrieve parallel sentences in Danish and Bornholmsk",
            "Norwegian courts": "Retrieve parallel sentences in Norwegian Bokmål and Nynorsk",
        }
        default_instruction = "Retrieve parallel sentences."
        return task_name_to_instruct.get(task.name, default_instruction)
    if task.task_type in ["Classification"]:
        task_name_to_instruct: dict[str, str] = {
            "Angry Tweets": "Classify Danish tweets by sentiment. (positive, negative, neutral)",
            "DKHate": "Classify Danish tweets based on offensiveness (offensive, not offensive)",
            "Da Political Comments": "Classify Danish political comments for sentiment",
            "DaLAJ": "Classify texts based on linguistic acceptability in Swedish",
            "LCC": "Classify texts based on sentiment",
            "Language Identification": "Classify texts based on language",
            "Massive Intent": "Given a user utterance as query, find the user intents",
            "Massive Scenario": "Given a user utterance as query, find the user scenarios",
            "NoReC": "Classify Norwegian reviews by sentiment",
            "SweReC": "Classify Swedish reviews by sentiment",
            "Norwegian parliament": "Classify parliament speeches in Norwegian based on political affiliation",
            "ScaLA": "Classify passages in Scandinavian Languages based on linguistic acceptability",
        }
        default_instruction = "Classify user passages"
        return task_name_to_instruct.get(task.name, default_instruction)
    if task.task_type in ["Clustering"]:
        task_name_to_instruct: dict[str, str] = {
            "ArxivClusteringP2P": "Identify the main and secondary category of Arxiv papers based on the titles and abstracts",
            "VG Clustering": "Identify the categories (e.g. sports) of given articles in Norwegian",
            "SNL Clustering": "Identify categories in a Norwegian lexicon",
            "SwednClustering": "Identify news categories in Swedish passages",
        }
        default_instruction = "Identify categories in user passages"
        return task_name_to_instruct.get(task.name, default_instruction)
    if task.task_type in ["Reranking"]:
        return "Retrieve semantically similar passages."
    if task.task_type in ["Retrieval"]:
        task_name_to_instruct: dict[str, str] = {
            "Twitterhjerne": "Retrieve answers to questions asked in Danish tweets",
            "SwednRetrieval": "Retrieve summaries of Swedish news articles",
            "TV2Nord Retrieval": "Retrieve summaries of Danish news articles",
            "DanFEVER": "Given a claim in Danish, retrieve documents that support or refute the claim",
            "SNL Retrieval": "Given a lexicon article in Norwegian, retrieve its headline",
            "NorQuad": "Given a question in Norwegian, retrieve the answer from Wikipedia articles",
            "SweFAQ": "Retrieve answers given questions in Swedish",
            "ArguAna": "Given a claim, find documents that refute the claim",
            "ClimateFEVER": "Given a claim about climate change, retrieve documents that support or refute the claim",
        }
        default_instruction = "Retrieve text based on user query."
        return task_name_to_instruct.get(task.name, default_instruction)
    return ""


class E5Mistral(Encoder):
    max_length = 4096

    def __init__(self):
        self.load_model()

    def load_model(self):
        self.tokenizer = AutoTokenizer.from_pretrained("intfloat/e5-mistral-7b-instruct")
        self.model = AutoModel.from_pretrained("intfloat/e5-mistral-7b-instruct")

    def preprocess(self, sentences: Sequence[str], instruction: str) -> BatchEncoding:
        sentences = [f"Instruction: {instruction} Query: {sentence}" for sentence in sentences]
        batch_dict = self.tokenizer(
            sentences,
            max_length=self.max_length - 1,
            return_attention_mask=False,
            padding=False,
            truncation=True,
        )
        # append eos_token_id to every input_ids
        batch_dict["input_ids"] = [
            [*input_ids, self.tokenizer.eos_token_id]
            for input_ids in batch_dict["input_ids"]  # type: ignore
        ]
        batch_dict = self.tokenizer.pad(batch_dict, padding=True, return_attention_mask=True, return_tensors="pt")

        return batch_dict

    # but it does not work slightly better than this:
    # return sentences # noqa

    @staticmethod
    def last_token_pool(last_hidden_states: Tensor, attention_mask: Tensor) -> Tensor:
        left_padding = attention_mask[:, -1].sum() == attention_mask.shape[0]
        if left_padding:
            return last_hidden_states[:, -1]
        sequence_lengths = attention_mask.sum(dim=1) - 1
        batch_size = last_hidden_states.shape[0]
        return last_hidden_states[
            torch.arange(batch_size, device=last_hidden_states.device),
            sequence_lengths,
        ]

    def encode(
        self,
        sentences: list[str],
        *,
        task: Optional[Task] = None,
        batch_size: int = 8,
        **kwargs: Any,  # noqa
    ) -> ArrayLike:
        batched_embeddings = []
        if task is not None:  # noqa
            instruction = task_to_instruction(task)
        else:
            instruction = ""
        for batch in batched(sentences, batch_size):
            batch_dict = self.preprocess(batch, instruction=instruction)

            outputs = self.model(**batch_dict)
            embeddings = self.last_token_pool(
                outputs.last_hidden_state,
                batch_dict["attention_mask"],  # type: ignore
            )

            # normalize embeddings
            embeddings = F.normalize(embeddings, p=2, dim=1)
            scores = (embeddings[:2] @ embeddings[2:].T) * 100
            batched_embeddings.append(scores)

        return torch.cat(batched_embeddings)


@models.register("intfloat/e5-mistral-7b-instruct")
def create_multilingual_e5_mistral_7b_instruct() -> SebModel:
    hf_name = "intfloat/e5-mistral-7b-instruct"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=[],
        open_source=True,
        embedding_size=4096,
        model_type="Mistral",
        release_date=date(2023, 12, 20),
    )
    return SebModel(
        encoder=LazyLoadEncoder(E5Mistral),
        meta=meta,
    )



from seb.interfaces.mteb_task import MTEBTask
from seb.interfaces.task import Task
from seb.mteb_tasks import NorwegianCourtsBitextMining, NorwegianParliamentClassification
from seb.registries import tasks


@tasks.register("NoReC")
def create_norec() -> Task:
    from mteb import NoRecClassification

    task = MTEBTask(NoRecClassification())
    task.name = "NoReC"
    task.domain = ["reviews"]
    return task


@tasks.register("Norwegian parliament")
def create_norwegian_parliament() -> Task:
    task = MTEBTask(NorwegianParliamentClassification())
    task.name = "Norwegian parliament"
    task.domain = ["spoken"]
    return task


@tasks.register("Norwegian courts")
def create_norwegian_courts() -> Task:
    task = MTEBTask(NorwegianCourtsBitextMining())
    task.name = "Norwegian courts"
    task.domain = ["legal", "non-fiction"]
    task._text_columns = ["sentence1", "sentence2"]
    return task


@tasks.register("VG Clustering")
def create_vg_clustering() -> Task:
    from seb.mteb_tasks import VGClustering

    task = MTEBTask(VGClustering())
    task.name = "VG Clustering"
    task.version = "0.0.1"
    task.domain = ["non-fiction", "news"]
    task._text_columns = ["sentences"]
    return task


@tasks.register("SNL Clustering")
def create_sts_clustering() -> Task:
    from seb.mteb_tasks import SNLClustering

    task = MTEBTask(SNLClustering())
    task.name = "SNL Clustering"
    task.version = "0.0.1"
    task.domain = ["non-fiction", "wiki"]
    task._text_columns = ["sentences"]
    return task


@tasks.register("SNL Retrieval")
def create_sts_retrieval() -> Task:
    from seb.mteb_tasks import SNLRetrieval

    task = MTEBTask(SNLRetrieval())
    task.name = "SNL Retrieval"
    task.version = "0.0.1"
    task.domain = ["non-fiction", "wiki"]
    return task


@tasks.register("NorQuad")
def create_norquad() -> Task:
    from seb.mteb_tasks import NorQuadRetrieval

    task = MTEBTask(NorQuadRetrieval())
    task.name = "NorQuad"
    task.version = "0.0.1"
    task.domain = ["non-fiction", "wiki"]
    return task



from typing import Any

import datasets
from mteb.abstasks import AbsTaskBitextMining


class NorwegianCourtsBitextMining(AbsTaskBitextMining):
    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "NorwegianCourtsBitextMining",
            "hf_hub_name": "kardosdrur/norwegian-courts",
            "description": "Nynorsk and Bokmål parallel corpus from Norwegian courts. "
            + "Norway has two standardised written languages. "
            + "Bokmål is a variant closer to Danish, while Nynorsk was created to resemble "
            + "regional dialects of Norwegian.",
            "reference": "https://opus.nlpl.eu/ELRC-Courts_Norway-v1.php",
            "type": "BitextMining",
            "category": "s2s",
            "eval_splits": ["test"],
            "eval_langs": ["nb", "nn"],
            "main_score": "f1",
            "revision": "d79af07e969a6678fcbbe819956840425816468f",
        }

    def load_data(self, **kwargs: Any) -> None:  # noqa: ARG002
        """
        Load dataset from HuggingFace hub and convert it to the standard format.
        """
        if self.data_loaded:
            return

        self.dataset = datasets.load_dataset(
            self.description["hf_hub_name"],
            revision=self.description.get("revision", None),
        )
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        # Convert to standard format
        self.dataset = self.dataset.rename_column("nb", "sentence1")
        self.dataset = self.dataset.rename_column("nn", "sentence2")



import json
from collections.abc import Iterable, Iterator
from datetime import datetime
from pathlib import Path
from typing import Optional, Union

import numpy as np
from pydantic import BaseModel

from .interfaces.language import Language
from .interfaces.model import ModelMeta


class TaskResult(BaseModel):
    """
    Dataclass for storing task results.

    Attributes:
        task_name: Name of the task.
        task_description: Description of the task.
        task_version: Version of the task.
        time_of_run: Time of the run.
        scores: Dictionary of scores on the form {language: {"metric": value}}.
        main_score: Name of the main score.
    """

    task_name: str
    task_description: str
    task_version: str
    time_of_run: datetime
    scores: dict[Language, dict[str, Union[float, str]]]  # {language: {"metric": value}}.
    main_score: str

    def get_main_score(self, lang: Optional[Iterable[str]] = None) -> float:
        """
        Returns the main score for a given set of languages.

        Args:
            lang: List of languages to get the main score for.

        Returns:
            The main score.
        """
        main_scores = []
        if lang is None:
            lang = self.scores.keys()

        for l in lang:
            main_scores.append(self.scores[l][self.main_score])  # type: ignore

        return sum(main_scores) / len(main_scores)

    @property
    def languages(self) -> list[Language]:
        """
        Returns the languages of the task.
        """
        return list(self.scores.keys())

    @classmethod
    def from_disk(cls, path: Path) -> "TaskResult":
        """
        Load task results from a path.
        """
        with path.open("r") as f:
            task_results = json.load(f)
        return cls(**task_results)

    def to_disk(self, path: Path) -> None:
        """
        Write task results to a path.
        """
        path.parent.mkdir(parents=True, exist_ok=True)
        json_str: str = self.model_dump_json()  # type: ignore

        with path.open("w") as f:
            f.write(json_str)

    def name_to_path(self) -> str:
        """
        Convert a name to a path.
        """
        name = self.task_name.replace("/", "__").replace(" ", "_")
        return name


class TaskError(BaseModel):
    task_name: str
    error: str
    time_of_run: datetime
    languages: list[str] = []

    def to_disk(self, path: Path) -> None:
        """
        Write task results to a path.
        """
        path.parent.mkdir(parents=True, exist_ok=True)
        json_str: str = self.model_dump_json()  # type: ignore

        with path.open("w") as f:
            f.write(json_str)

    @classmethod
    def from_disk(cls, path: Path) -> "TaskError":
        """
        Load task results from a path.
        """
        with path.open() as f:
            task_results = json.load(f)
        return cls(**task_results)

    @staticmethod
    def get_main_score(lang: Optional[Iterable[str]] = None) -> float:  # noqa: ARG004
        return np.nan

    def name_to_path(self) -> str:
        """
        Convert a name to a path.
        """
        name = self.task_name.replace("/", "__").replace(" ", "_")
        return name


class BenchmarkResults(BaseModel):
    """
    Dataclass for storing benchmark results.

    Attributes:
        meta: ModelMeta object.
        task_results: List of TaskResult objects.
    """

    meta: ModelMeta
    task_results: list[Union[TaskResult, TaskError]]

    def get_main_score(self, lang: Optional[Iterable[Language]] = None) -> float:
        scores = [t.get_main_score(lang) for t in self.task_results]
        if scores:
            return sum(scores) / len(scores)
        return np.nan

    def __iter__(self) -> Iterator[Union[TaskResult, TaskError]]:  # type: ignore
        return iter(self.task_results)

    def __getitem__(self, index: int) -> Union[TaskResult, TaskError]:
        return self.task_results[index]

    def __len__(self) -> int:
        return len(self.task_results)

    def to_disk(self, path: Path) -> None:
        """
        Write task results to a path.
        """
        if path.is_file():
            raise ValueError("Can't save BenchmarkResults to a file. Path must be a directory.")
        path.mkdir(parents=True, exist_ok=True)
        for task_result in self.task_results:
            if isinstance(task_result, TaskResult):
                task_result.to_disk(path / f"{task_result.task_name}.json")
            else:
                task_result.to_disk(path / f"{task_result.task_name}.error.json")

        meta_path = path / "meta.json"
        self.meta.to_disk(meta_path)

    @classmethod
    def from_disk(cls, path: Path) -> "BenchmarkResults":
        """
        Load task results from a path.
        """
        if not path.is_dir():
            raise ValueError("Can't load BenchmarkResults from path: {path}. Path must be a directory.")
        task_results = []
        for file in path.glob("*.json"):
            if file.stem == "meta":
                continue
            if file.stem.endswith(".error"):
                task_results.append(TaskError.from_disk(file))
            else:
                task_results.append(TaskResult.from_disk(file))

        meta_path = path / "meta.json"
        meta = ModelMeta.from_disk(meta_path)
        return cls(meta=meta, task_results=task_results)



import seb


def test_all_model_are_unique():
    models = seb.get_all_models()

    # model name should be unique
    model_names = [model.meta.name for model in models]
    assert len(model_names) == len(set(model_names))

    # cache path should be unique
    cache_paths = [model.meta.get_path_name() for model in models]
    assert len(cache_paths) == len(set(cache_paths))



from collections.abc import Sequence
from functools import partial
from typing import Any, Optional

import torch
from transformers import M2M100ForConditionalGeneration, M2M100Tokenizer

from seb.interfaces.model import Encoder, LazyLoadEncoder, ModelMeta, SebModel
from seb.interfaces.task import Task
from seb.registered_models.e5_models import E5Wrapper
from seb.registries import models


class TranslateE5Model(Encoder):
    def __init__(self, model_name: str) -> None:
        self.model_name = model_name
        self.mdl = E5Wrapper(model_name)
        self.trans_model: M2M100ForConditionalGeneration = (  # type: ignore
            M2M100ForConditionalGeneration.from_pretrained("facebook/m2m100_418M")
        )
        self.trans_tokenizer = M2M100Tokenizer.from_pretrained("facebook/m2m100_418M")

    def translate(self, sentence: str, src_lang: str) -> str:
        self.trans_tokenizer.src_lang = src_lang
        encoded_sent = self.trans_tokenizer(sentence, return_tensors="pt")
        gen_tokens = self.trans_model.generate(**encoded_sent, forced_bos_token_id=self.trans_tokenizer.get_lang_id("en"))
        return self.trans_tokenizer.batch_decode(gen_tokens, skip_special_tokens=True)[0]

    def encode(
        self,
        sentences: list[str],
        *,
        task: Optional[Task] = None,
        batch_size: int = 32,
        **kwargs: Any,
    ) -> torch.Tensor:
        if task:
            try:
                src_lang = task.languages[0]  # type: ignore
            except IndexError:
                # Danish is the default fallback if no language is specified for the task.
                src_lang = "da"
        else:
            src_lang = "da"
        sentences = [self.translate(sentence, src_lang) for sentence in sentences]
        return self.mdl.encode(sentences, task=task, batch_size=batch_size, **kwargs)  # type: ignore


@models.register("translate-e5-small")
def create_translate_e5_small() -> SebModel:
    hf_name = "intfloat/e5-small"
    meta = ModelMeta(
        name="translate-e5-small",
        reference=f"https://huggingface.co/{hf_name}",
        languages=["en"],
        open_source=True,
        embedding_size=384,
        model_type="Translate-Embed",
        release_date=None,
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(TranslateE5Model, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("translate-e5-base")
def create_translate_e5_base() -> SebModel:
    hf_name = "intfloat/e5-base"
    meta = ModelMeta(
        name="translate-e5-base",
        reference=f"https://huggingface.co/{hf_name}",
        languages=["en"],
        open_source=True,
        embedding_size=384,
        model_type="Translate-Embed",
        release_date=None,
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(TranslateE5Model, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("translate-e5-large")
def create_translate_e5_large() -> SebModel:
    hf_name = "intfloat/e5-large"
    meta = ModelMeta(
        name="translate-e5-large",
        reference=f"https://huggingface.co/{hf_name}",
        languages=["en"],
        open_source=True,
        embedding_size=384,
        model_type="Translate-Embed",
        release_date=None,
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(TranslateE5Model, model_name=hf_name)),  # type: ignore
        meta=meta,
    )



"""
This is the specification for the full benchmark. Running the code here will reproduce the results.
"""


import logging
from pathlib import Path
from typing import Optional

from .benchmark import Benchmark
from .interfaces.model import SebModel
from .registered_tasks.speed import CPUSpeedTask, GPUSpeedTask
from .registries import get_all_models
from .result_dataclasses import BenchmarkResults

logger = logging.getLogger(__name__)

BENCHMARKS = {
    "Mainland Scandinavian": ["da", "sv", "nn", "nb"],
    "Danish": ["da"],
    "Norwegian": ["nn", "nb"],
    "Swedish": ["sv"],
}


def run_benchmark(
    use_cache: bool = True,
    run_models: bool = True,
    raise_errors: bool = True,
    cache_dir: Optional[Path] = None,
) -> dict[str, list[BenchmarkResults]]:
    """
    Run the full SEB benchmark.
    """
    models: list[SebModel] = get_all_models()

    results = {}
    for subset, langs in BENCHMARKS.items():
        benchmark = Benchmark(languages=langs)
        bm_results = benchmark.evaluate_models(
            models=models,
            use_cache=use_cache,
            run_model=run_models,
            raise_errors=raise_errors,
            cache_dir=cache_dir,
        )

        results[subset] = bm_results

    return results


def run_speed_benchmark(
    use_cache: bool = True,
    run_models: bool = True,
    raise_errors: bool = True,
    cache_dir: Optional[Path] = None,
) -> dict[str, list[BenchmarkResults]]:
    """
    Run the speed benchmark.
    """
    models: list[SebModel] = get_all_models()
    tasks = [CPUSpeedTask()]  # type: ignore

    if use_cache:
        logger.warn(
            "Running the speed benchmark with use_cache=True will load speed results from the cache, this might lead to incomparable results."
        )

    results = {}
    for subset, langs in BENCHMARKS.items():
        benchmark = Benchmark(languages=langs, tasks=tasks)
        bm_results = benchmark.evaluate_models(
            models=models,
            use_cache=use_cache,
            run_model=run_models,
            raise_errors=raise_errors,
            cache_dir=cache_dir,
        )

        results[subset] = bm_results

    return results



from datetime import datetime
from functools import partial
from typing import Any, Union

import numpy as np
from datasets import DatasetDict, concatenate_datasets
from mteb import AbsTask
from mteb import __version__ as mteb_version

from ..result_dataclasses import TaskResult
from .model import Encoder
from .task import DescriptiveDatasetStats, Task


class MTEBTask(Task):
    def __init__(self, mteb_task: AbsTask) -> None:
        self.mteb_task = mteb_task
        mteb_desc = self.mteb_task.description
        self.main_score = mteb_desc["main_score"]
        self.name = mteb_desc["name"]
        self.description = mteb_desc["description"]
        self.version = f"{mteb_version}"
        self.reference = mteb_desc["reference"]
        self.languages = mteb_desc["eval_langs"]
        self.task_type = mteb_desc["type"]
        self.domain = []
        self._text_columns = ["text"]

    def load_data(self) -> DatasetDict:
        self.mteb_task.load_data()
        ds = {}
        # check if it is split into dataset[lang][split] or to dataset[split]
        keys = list(self.mteb_task.dataset.keys())  # type: ignore
        contains_lang = isinstance(self.mteb_task.dataset[keys[0]], DatasetDict)  # type: ignore

        if not contains_lang:  # it is split into dataset[split]
            for split in self.mteb_task.description["eval_splits"]:
                ds[split] = self.mteb_task.dataset[split]  # type: ignore
        else:  # it is split into dataset[lang][split] and we convert to dataset[split]
            for lang in self.languages:
                for split in self.mteb_task.description["eval_splits"]:
                    if split not in ds:
                        ds[split] = self.mteb_task.dataset[lang][split]  # type: ignore
                    else:
                        ds[split] = concatenate_datasets(
                            [ds[split], self.mteb_task.dataset[lang][split]],  # type: ignore
                        )

        return DatasetDict(ds)

    def get_documents(self) -> list[str]:
        ds: DatasetDict = self.load_data()
        texts = []
        splits = self.mteb_task.description["eval_splits"]
        assert len(splits) >= 1, "No splits found in MTEB task."

        for split in splits:
            if self.task_type == "Retrieval":
                _corpus = self.mteb_task.corpus[split]  # type: ignore
                _queries = self.mteb_task.queries[split]  # type: ignore
                texts = [f"{text['title']} {text['text']}" for text in _corpus.values()]
                texts += list(_queries.values())
            elif self.task_type == "Clustering":
                for text_column in self._text_columns:
                    texts += [text for tl in ds[split][text_column] for text in tl]
            else:
                for text_column in self._text_columns:
                    texts += ds[split][text_column]

        return texts

    def get_descriptive_stats(self) -> DescriptiveDatasetStats:
        texts = self.get_documents()
        document_lengths = np.array([len(text) for text in texts])

        mean = float(np.mean(document_lengths))
        std = float(np.std(document_lengths))
        return DescriptiveDatasetStats(
            mean_document_length=mean,
            std_document_length=std,
            num_documents=len(document_lengths),
        )

    def format_scores(self, raw_scores: dict[str, Any], split: str) -> dict[str, Any]:
        if self.task_type == "STS":
            raw_scores = {
                "cosine_spearman": raw_scores["cos_sim"]["spearman"],
                "cosine_pearson": raw_scores["cos_sim"]["pearson"],
                "euclidean_spearman": raw_scores["euclidean"]["spearman"],
                "euclidean_pearson": raw_scores["euclidean"]["pearson"],
                "manhattan_spearman": raw_scores["manhattan"]["spearman"],
                "manhattan_pearson": raw_scores["manhattan"]["pearson"],
            }

        raw_scores = raw_scores.get(split, raw_scores)
        score_is_nested = isinstance(raw_scores[next(iter(raw_scores.keys()))], dict)
        if not score_is_nested:
            _scores: dict[str, dict[str, Union[float, str]]] = {lang: raw_scores for lang in self.languages}
            scores = _scores
        else:
            scores = raw_scores

        return scores

    def evaluate(self, model: Encoder) -> TaskResult:
        split = self.mteb_task.description["eval_splits"][0]
        # Infusing task into encode()
        original_encode = model.encode
        try:
            model.encode = partial(model.encode, task=self)
            scores = self.mteb_task.evaluate(model, split=split)
        except Exception as e:
            raise e
        finally:
            # Resetting encode to original
            model.encode = original_encode
        if scores is None:
            raise ValueError("MTEBTask evaluation failed.")

        # there is only one split in all MTEB tasks in SEB

        time_of_run: datetime = datetime.now()

        scores = self.format_scores(scores, split)

        task_result = TaskResult(
            task_name=self.name,
            task_description=self.description,
            task_version=self.version,
            time_of_run=time_of_run,
            scores=scores,  # type: ignore
            main_score=self.main_score,
        )

        return task_result



from datetime import datetime
from typing import Any

import seb
from seb.registries import tasks


@tasks.register("test-encode-task")
def create_test_encode_task() -> seb.Task:
    class DummyTask(seb.Task):
        name = "test-encode-task"
        main_score = "a_metric"
        description = "NA"
        reference = "NA"
        version = "NA"
        languages = []  # noqa: RUF012
        domain = []  # noqa: RUF012
        task_type = "Classification"

        def evaluate(self, model: seb.Encoder) -> seb.TaskResult:
            model.encode(["a test sentence"], task=self)

            return seb.TaskResult(
                task_name="test-encode-task",
                task_description="NA",
                task_version="NA",
                time_of_run=datetime.now(),
                scores={"en": {"a_metric": 1.0}},
                main_score="a_metric",
            )

        def get_descriptive_stats(self) -> dict[str, Any]:
            return {}

    return DummyTask()


import numpy as np

import seb
from seb.registries import models


@models.register("test_model")
def create_test_model() -> seb.Encoder:
    class TestEncoder:
        def encode(
            self,
            sentences: list[str],
            batch_size: int,  # noqa: ARG002
            **kwargs: dict,  # noqa: ARG002
        ) -> np.ndarray:
            # create random array of 100, pr text
            return np.array([np.random.rand(100) for _ in sentences])

    def load_test_model() -> TestEncoder:
        return TestEncoder()

    assert isinstance(TestEncoder, seb.Encoder)

    return seb.SebModel(
        meta=seb.ModelMeta(name="test_model", embedding_size=100),
        encoder=seb.LazyLoadEncoder(load_test_model),  # type: ignore
    )



# This just created a rough draft of the CLI documentation. It is not
# intended to be used for anything other than a starting point.
# at least we would need this issue fixed first:
# https://github.com/explosion/radicli/issues/30

from pathlib import Path

from seb.cli import cli

title = "Command Line Interface"
description = "Documentation for the command line interface of SEB."

if __name__ == "__main__":
    with Path("docs/cli.md").open("w", encoding="utf8") as f:
        f.write(cli.document(title=title, description=description))



import random
from typing import Any, TypeVar

import datasets
from mteb.abstasks import AbsTaskSTS
from seb.interfaces.mteb_task import MTEBTask
from seb.interfaces.task import Task
from seb.registries import tasks

T = TypeVar("T")


class SwednSummarizationSTS(AbsTaskSTS):
    def load_data(self, **kwargs: dict):  # noqa: ARG002
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset: datasets.DatasetDict = datasets.load_dataset(
            self.description["hf_hub_name"],
            "swedn",  # chose the relevant subset
            revision=self.description.get("revision"),
        )  # type: ignore

        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        self.dataset = self.dataset.rename_column("summary", "sentence2")
        self.dataset = self.dataset.rename_column("article", "sentence1")
        self.dataset = self.dataset.remove_columns(["id", "headline", "article_category"])
        self.dataset = self.dataset.shuffle(seed=42)

        # add score column
        for split in self.dataset:
            ds_split = self.dataset[split]
            ds_split = ds_split.add_column("score", [1] * len(ds_split))  # type: ignore
            self.dataset[split] = ds_split

            # Add a wrongly mapped examples. To ensure tasks in non-trivial
            summaries = ds_split["sentence2"]
            articles = ds_split["sentence1"]
            scores = ds_split["score"]
            mismatched_summaries = self.sattolo_cycle(summaries)

            # add all the mismatched examples as negative examples
            mismatched_ds = datasets.Dataset.from_dict(
                {
                    "sentence1": articles,
                    "sentence2": mismatched_summaries,
                    "score": ([0] * len(articles)),
                }
            )
            mismatched_ds = mismatched_ds.shuffle(seed=42)
            self.dataset[split] = datasets.concatenate_datasets([ds_split.select(range(1024)), mismatched_ds.select(range(1024))])

    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "Swedn",
            "hf_hub_name": "sbx/superlim-2",
            "description": "News Article Summary Semantic Similarity Estimation.",
            "reference": "https://spraakbanken.gu.se/en/resources/swedn",
            "type": "STS",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["sv"],
            "main_score": "cosine_spearman",
            "min_score": 0,
            "max_score": 1,
            "revision": "ef1661775d746e0844b299164773db733bdc0bf6",
        }

    @staticmethod
    def sattolo_cycle(items: list[T]) -> list[T]:
        """
        The Sattolo cycle is a simple algorithm for randomly shuffling an array in-place.
        It ensures that the element i, will not be in the ith position of the result.
        """
        rng = random.Random(42)
        for i in range(len(items) - 1, 0, -1):
            j = rng.randint(0, i - 1)
            items[i], items[j] = items[j], items[i]
        return items


@tasks.register("SwednSTS")
def create_swedn_sts() -> Task:
    task = MTEBTask(SwednSummarizationSTS())
    task.name = "SwednSTS"
    task.version = "0.0.1"
    task.domain = ["non-fiction", "news"]
    return task



"""
Experiments:

Using two sample models (text-embedding-3-small, all-MiniLM-L6-v2) we get the following results:

96.07, 64.67

if we then remove the ingress from the corpus we get:
92.71, 51.61
The reason why we might want to remove the ingress is that it almost always start with headline.

As the scores are indeed slightly lower we will ignore the ingress as the task becomes too easy.
"""

from typing import Any

import datasets
from mteb.abstasks import AbsTaskRetrieval


class SNLRetrieval(AbsTaskRetrieval):
    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "SNLClustering",
            "hf_hub_name": "navjordj/SNL_summarization",
            "description": "Webscrabed articles and ingresses from the Norwegian lexicon 'Det Store Norske Leksikon'.",
            "reference": "https://huggingface.co/datasets/navjordj/SNL_summarization",
            "type": "Retrieval",
            "category": "p2p",
            "eval_splits": ["test"],
            "eval_langs": ["nb"],
            "main_score": "ndcg_at_10",
            "revision": "3d3d27aa7af8941408cefc3991ada5d12a4273d1",
        }

    def load_data(self, **kwargs: dict):  # noqa: ARG002
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset: datasets.DatasetDict = datasets.load_dataset(
            self.description["hf_hub_name"],
            revision=self.description.get("revision"),
        )  # type: ignore

        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """
        and transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}
        text2id = {}

        for split in self.dataset:
            ds: datasets.Dataset = self.dataset[split]  # type: ignore
            ds = ds.shuffle(seed=42)

            self.queries[split] = {}
            self.relevant_docs[split] = {}
            self.corpus[split] = {}

            headline = ds["headline"]
            article = ds["article"]

            n = 0
            for headl, art in zip(headline, article):
                self.queries[split][str(n)] = headl
                q_n = n
                n += 1
                if art not in text2id:
                    text2id[art] = n
                    self.corpus[split][str(n)] = {"title": "", "text": art}
                    n += 1
                self.relevant_docs[split][str(q_n)] = {str(text2id[art]): 1}  # only one correct matches



import logging
import os
from collections.abc import Iterable
from datetime import datetime
from pathlib import Path
from typing import Optional, Union

from tqdm import tqdm

from .interfaces.model import Encoder, SebModel
from .interfaces.task import Task
from .registries import get_all_tasks, get_task
from .result_dataclasses import BenchmarkResults, TaskError, TaskResult
from .warning_ignore_manager import WarningIgnoreContextManager

logger = logging.getLogger(__name__)


package_dir = Path(__file__).parent
CACHE_DIR = package_dir / "cache"


def get_cache_dir() -> Path:
    """
    Get the cache directory for SEB. Can be overridden by setting the environment
    variable SEB_CACHE_DIR.
    """
    cache_dir = os.environ.get("SEB_CACHE_DIR")
    if cache_dir is not None:
        return Path(cache_dir)
    return CACHE_DIR


def get_cache_path(task: Task, model: SebModel, cache_dir: Optional[Path] = None) -> Path:
    """
    Get the cache path for a task and model.
    """
    cache_path = cache_dir if cache_dir is not None else get_cache_dir()
    mdl_path_name = model.meta.get_path_name()
    task_path_name = task.name_to_path() + ".json"
    task_cache_path = cache_path / mdl_path_name / task_path_name
    return task_cache_path


def run_task(
    task: Task,
    model: SebModel,
    use_cache: bool,
    run_model: bool,
    raise_errors: bool,
    cache_dir: Optional[Path] = None,
) -> Union[TaskResult, TaskError]:
    """
    Tests a model on a task
    """
    if run_model is False and use_cache is False:
        raise ValueError("run_model and use_cache cannot both be False")

    if not raise_errors:
        try:
            return run_task(
                task=task,
                model=model,
                use_cache=use_cache,
                run_model=run_model,
                raise_errors=True,
                cache_dir=cache_dir,
            )
        except Exception as e:
            logger.error(f"Error when running {task.name} on {model.meta.name}: {e}")
            return TaskError(
                task_name=task.name,
                error=str(e),
                time_of_run=datetime.now(),
            )

    cache_path = get_cache_path(task, model, cache_dir)
    if cache_path.exists() and use_cache:
        logger.debug(f"Loading cached result for {model.meta.name} on {task.name}")
        task_result = TaskResult.from_disk(cache_path)
        return task_result

    cache_path.parent.mkdir(parents=True, exist_ok=True)

    if not run_model:
        raise ValueError(
            f"Cache for {model.meta.name} on {task.name} does not exist. " "Set run_model=True to run the model.",
        )
    with WarningIgnoreContextManager():
        task_result = task.evaluate(model.encoder)
    task_result.to_disk(cache_path)
    return task_result


class Benchmark:
    """
    Benchmark is the main orchestrator of the SEB benchmark.
    """

    def __init__(
        self,
        languages: Optional[list[str]] = None,
        tasks: Optional[Union[Iterable[str], Iterable[Task]]] = None,
    ) -> None:
        """
        Initialize the benchmark.

        Args:
            languages: A list of languages to run the benchmark on. If None, all languages are used.
            tasks: The tasks to run the benchmark on. If None, all tasks are used. Can either be specified as strings or as Task objects.
        """
        self.languages = languages

        self.tasks = self.get_tasks(tasks, languages)
        self.task_names = [task.name for task in self.tasks]

    @staticmethod
    def get_tasks(
        tasks: Optional[Union[Iterable[str], Iterable[Task]]],
        languages: Optional[list[str]],
    ) -> list[Task]:
        """
        Get the tasks for the benchmark.

        Returns:
            A list of tasks.
        """
        _tasks = []

        if tasks is None:
            _tasks = get_all_tasks()
        else:
            for task in tasks:
                if isinstance(task, str):
                    _tasks.append(get_task(task))
                elif isinstance(task, Task):
                    _tasks.append(task)
                else:
                    raise ValueError(f"Invalid task type: {type(task)}")

        if languages is not None:
            langs = set(languages)
            _tasks = [task for task in _tasks if set(task.languages) & langs]

        return _tasks

    def evaluate_model(
        self,
        model: SebModel,
        use_cache: bool = True,
        run_model: bool = True,
        raise_errors: bool = True,
        cache_dir: Optional[Path] = None,
        verbose: bool = True,
    ) -> BenchmarkResults:
        """
        Evaluate a model on the benchmark.

        Args:
            model: The model to evaluate.
            use_cache: Whether to use the cache.
            run_model: Whether to run the model if the cache is not present.
            raise_errors: Whether to raise errors.
            cache_dir: The cache directory to use. If None, the default cache directory is used.
            verbose: Whether to show a progress bar.

        Returns:
            The results of the benchmark.
        """
        task_results = []
        pbar = tqdm(
            self.tasks,
            position=1,
            desc=f"Running {model.meta.name}",
            leave=False,
            disable=not verbose,
        )
        for task in pbar:
            pbar.set_description(f"Running {model.meta.name} on {task.name}")
            task_result = run_task(
                task,
                model,
                use_cache=use_cache,
                run_model=run_model,
                raise_errors=raise_errors,
                cache_dir=cache_dir,
            )
            task_results.append(task_result)

        return BenchmarkResults(meta=model.meta, task_results=task_results)

    def evaluate_models(
        self,
        models: list[SebModel],
        use_cache: bool = True,
        run_model: bool = True,
        raise_errors: bool = True,
        cache_dir: Optional[Path] = None,
        verbose: bool = True,
    ) -> list[BenchmarkResults]:
        """
        Evaluate a list of models on the benchmark.

        Args:
            models: The models to evaluate.
            use_cache: Whether to use the cache.
            run_model: Whether to run the model if the cache is not present.
            raise_errors: Whether to raise errors.
            cache_dir: The cache directory to use. If None, the default cache directory is used.
            verbose: Whether to show a progress bar.

        Returns:
            The results of the benchmark, once for each model.
        """
        results = []
        pbar = tqdm(
            models,
            position=0,
            desc="Running Benchmark",
            leave=True,
            disable=not verbose,
        )

        for model in pbar:
            pbar.set_description(f"Running {model.meta.name}")
            results.append(
                self.evaluate_model(
                    model,
                    use_cache=use_cache,
                    run_model=run_model,
                    raise_errors=raise_errors,
                    cache_dir=cache_dir,
                    verbose=verbose,
                ),
            )
        return results



from datetime import datetime
from pathlib import Path

import numpy as np
import seb

from .dummy_task import TestTask


def create_test_model_with_task_dependent_encode() -> seb.SebModel:
    class TestEncoder(seb.Encoder):
        def encode(
            self,
            sentences: list[str],
            *,
            task: seb.Task,
            batch_size: int,  # noqa: ARG002
        ) -> np.ndarray:
            if task.task_type == "SNS":
                return np.array([np.ones(100) for _ in sentences])
            return np.array([np.zeros(100) for _ in sentences])

    def load_test_model() -> TestEncoder:
        return TestEncoder()

    return seb.SebModel(
        meta=seb.ModelMeta(name="test_model_with_task_dependent_encode", embedding_size=100),
        encoder=seb.LazyLoadEncoder(load_test_model),
    )


def create_all_is_0_task() -> seb.Task:
    class TestTaskAllEmbeddingIsOne(TestTask):
        name = "embeddings is one task"
        task_type: str = "SNS"

        def evaluate(self, model: seb.Encoder) -> seb.TaskResult:
            out = model.encode(["a test sentence"], task=self)
            assert np.all(out == 1)

            return seb.TaskResult(
                task_name=self.name,
                task_description="NA",
                task_version="NA",
                time_of_run=datetime.now(),
                scores={"nb": {"a_metric": 1.0}},
                main_score="a_metric",
            )

    return TestTaskAllEmbeddingIsOne()


def create_all_is_1_task() -> seb.Task:
    class TestTaskAllEmbeddingIsZero(TestTask):
        name = "all embeddings is 0 task"
        task_type = "Classification"

        def evaluate(self, model: seb.Encoder) -> seb.TaskResult:
            out = model.encode(["a test sentence"], task=self)
            assert np.all(out == 0)

            return seb.TaskResult(
                task_name=self.name,
                task_description="NA",
                task_version="NA",
                time_of_run=datetime.now(),
                scores={"nb": {"a_metric": 1.0}},
                main_score="a_metric",
            )

    return TestTaskAllEmbeddingIsZero()


def test_task_dependent_encode(tmp_path: Path):
    model = create_test_model_with_task_dependent_encode()

    tasks = [
        create_all_is_0_task(),
        create_all_is_1_task(),
    ]

    benchmark = seb.Benchmark(tasks=tasks)
    result = benchmark.evaluate_model(model, cache_dir=tmp_path)
    assert result.get_main_score() == 1, "both datasets should have score of 1 if they run successfully"



from collections.abc import Sequence
from datetime import date
from functools import partial
from typing import Any

import numpy as np
import torch

from seb.interfaces.model import Encoder, LazyLoadEncoder, ModelMeta, SebModel
from seb.registries import models


class FastTextModel(Encoder):
    def __init__(self, model_name: str, lang: str) -> None:
        self.model_name = model_name
        self.lang = lang

        import fasttext  # type: ignore
        import fasttext.util  # type: ignore

        fasttext.util.download_model(self.lang, if_exists="ignore")
        self.model = fasttext.load_model(self.model_name)

    def get_embedding_dim(self) -> int:
        v = self.encode(["get emb dim"])
        return v.shape[1]

    def encode(  # type: ignore
        self,
        sentences: Sequence[str],
        **kwargs: Any,  # noqa: ARG002
    ) -> torch.Tensor:
        embeddings = []
        for sentence in sentences:
            # This is to appease FastText as they made the function err
            # if there's a \n in the sentence.
            sentence = " ".join(sentence.split())  # noqa
            sentence_embedding = self.model.get_sentence_vector(sentence)
            embeddings.append(sentence_embedding)
        return torch.tensor(np.stack(embeddings))


@models.register("fasttext-cc-da-300")
def create_cc_da_300() -> SebModel:
    model_name = "fasttext-cc-da-300"
    meta = ModelMeta(
        name=model_name,
        huggingface_name=None,
        reference="https://fasttext.cc/docs/en/crawl-vectors.html",
        languages=["da"],
        open_source=True,
        embedding_size=300,
        model_type="fastText",
        release_date=date(2017, 1, 1),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(FastTextModel, model_name="cc.da.300.bin", lang="da")),
        meta=meta,
    )


@models.register("fasttext-cc-sv-300")
def create_cc_sv_300() -> SebModel:
    model_name = "fasttext-cc-sv-300"
    meta = ModelMeta(
        name=model_name,
        huggingface_name=None,
        reference="https://fasttext.cc/docs/en/crawl-vectors.html",
        languages=["sv"],
        open_source=True,
        embedding_size=300,
        model_type="fastText",
        release_date=date(2017, 1, 1),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(FastTextModel, model_name="cc.sv.300.bin", lang="sv")),
        meta=meta,
    )


@models.register("fasttext-cc-nb-300")
def create_cc_nb_300() -> SebModel:
    model_name = "fasttext-cc-nb-300"
    meta = ModelMeta(
        name=model_name,
        huggingface_name=None,
        reference="https://fasttext.cc/docs/en/crawl-vectors.html",
        languages=["nb"],
        open_source=True,
        embedding_size=300,
        model_type="fastText",
        release_date=date(2017, 1, 1),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(FastTextModel, model_name="cc.no.300.bin", lang="no")),
        meta=meta,
    )


@models.register("fasttext-cc-nn-300")
def create_cc_nn_300() -> SebModel:
    model_name = "fasttext-cc-nn-300"
    meta = ModelMeta(
        name=model_name,
        huggingface_name=None,
        reference="https://fasttext.cc/docs/en/crawl-vectors.html",
        languages=["nn"],
        open_source=True,
        embedding_size=300,
        model_type="fastText",
        release_date=date(2017, 1, 1),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(FastTextModel, model_name="cc.nn.300.bin", lang="nn")),
        meta=meta,
    )



"""
The openai embedding api's evaluated on the SEB benchmark.
"""


import logging
from datetime import date
from functools import partial
from typing import Any, Optional

import torch

from seb.interfaces.model import Encoder, LazyLoadEncoder, ModelMeta, SebModel
from seb.interfaces.task import Task
from seb.registries import models

logger = logging.getLogger(__name__)


class CohereTextEmbeddingModel(Encoder):
    def __init__(self, model_name: str, sep: str = " ") -> None:
        self.model_name = model_name
        self.sep = sep

    def get_embedding_dim(self) -> int:
        v = self._embed(["get emb dim"], input_type="classification")
        return v.shape[1]

    def _embed(self, sentences: list[str], input_type: str) -> torch.Tensor:
        import cohere

        client = cohere.Client()
        response = client.embed(
            texts=list(sentences),
            model=self.model_name,
            input_type=input_type,
        )
        return torch.tensor(response.embeddings)

    def encode(
        self,
        sentences: list[str],
        batch_size: int = 32,  # noqa: ARG002
        *,
        task: Optional[Task] = None,
        **kwargs: Any,  # noqa: ARG002
    ) -> torch.Tensor:
        if task and task.task_type == "Classification":
            input_type = "classification"
        elif task and task.task_type == "Clustering":
            input_type = "clustering"
        else:
            input_type = "search_document"
        return self._embed(sentences, input_type=input_type)

    def encode_queries(self, queries: list[str], batch_size: int, **kwargs):  # noqa
        return self._embed(queries, input_type="search_query")

    def encode_corpus(self, corpus: list[dict[str, str]], batch_size: int, **kwargs):  # noqa
        if isinstance(corpus, dict):
            sentences = [
                (corpus["title"][i] + self.sep + corpus["text"][i]).strip() if "title" in corpus else corpus["text"][i].strip()  # type: ignore
                for i in range(len(corpus["text"]))  # type: ignore
            ]
        else:
            sentences = [(doc["title"] + self.sep + doc["text"]).strip() if "title" in doc else doc["text"].strip() for doc in corpus]
        return self._embed(sentences, input_type="search_document")


@models.register("embed-multilingual-v3.0")
def create_embed_multilingual_v3() -> SebModel:
    model_name = "embed-multilingual-v3.0"
    meta = ModelMeta(
        name=model_name,
        huggingface_name=None,
        reference="https://huggingface.co/Cohere/Cohere-embed-multilingual-v3.0",
        languages=[],
        open_source=False,
        embedding_size=1024,
        model_type="API",
        release_date=date(2023, 11, 2),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(CohereTextEmbeddingModel, model_name=model_name)),
        meta=meta,
    )



from collections.abc import Sequence
from datetime import date
from functools import partial
from pathlib import Path
from typing import Any, Optional, Union

import numpy as np
import torch

from seb.interfaces.model import Encoder, LazyLoadEncoder, ModelMeta, SebModel
from seb.interfaces.task import Task
from seb.registries import models


def truncate_seq_length(  # noqa: ANN201
    sequence_batch,  # SequenceBatch ,  # noqa: ANN001
    max_seq_len: int = 514,
):  # -> SequenceBatch:
    sequence_batch.seqs = sequence_batch.seqs[:, :max_seq_len]
    sequence_batch.seq_lens = torch.clamp(sequence_batch.seq_lens, max=max_seq_len)  # type: ignore
    return sequence_batch


class SonarTextToEmbeddingModelPipeline(Encoder):
    def __init__(
        self,
        source_lang: str,
        device: Optional[torch.device] = None,
    ) -> None:
        """
        Args:
            encoder_name: Name of the encoder model
            tokenizer_name: Name of the tokenizer
            device: Defaults to cpu
            source_lang: Set source_lang to '[dan|swe|nno|nob|]_Latn' for Danish, Swedish,
                Norwegian Nynorsk, and Norwegian Bokmål, respectively.
        """
        from sonar.inference_pipelines.text import TextToEmbeddingModelPipeline  # type: ignore

        self.t2vec_model = TextToEmbeddingModelPipeline(encoder="text_sonar_basic_encoder", tokenizer="text_sonar_basic_encoder")

        self.to(device=device)
        self.source_lang = source_lang

    def to(self, device: Optional[torch.device]) -> "SonarTextToEmbeddingModelPipeline":
        if device is None:
            device = torch.device("cpu")
        self.model = self.t2vec_model.to(device)
        self.device = device
        return self

    @torch.inference_mode()
    def encode(  # type: ignore
        self,
        sentences: Union[Path, Sequence[str]],
        *,
        task: Optional[Task] = None,  # noqa: ARG002
        batch_size: int,
        **kwargs: Any,  # noqa: ARG002
    ) -> np.ndarray:
        sentence_embeddings = self.t2vec_model.predict(sentences, source_lang=self.source_lang, batch_size=batch_size)
        return sentence_embeddings.numpy()


def get_sonar_model(source_lang: str) -> SonarTextToEmbeddingModelPipeline:
    try:
        return SonarTextToEmbeddingModelPipeline(
            source_lang=source_lang,
        )
    except ImportError:
        msg = "Could not fetch Sonar Models. Make sure you have" + " fairseq2 installed. This is currently only supported for " + "Linux."
        raise ImportError(msg)  # noqa B904


@models.register("sonar-dan")
def create_sonar_da() -> SebModel:
    meta = ModelMeta(
        name="sonar-dan",
        huggingface_name=None,
        reference="https://github.com/facebookresearch/SONAR",
        languages=["da"],
        open_source=True,
        embedding_size=1024,
        model_type="SONAR",
        release_date=date(2023, 5, 17),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sonar_model, source_lang="dan_Latn")),
        meta=meta,
    )


@models.register("sonar-swe")
def create_sonar_sv() -> SebModel:
    meta = ModelMeta(
        name="sonar-swe",
        huggingface_name=None,
        reference="https://github.com/facebookresearch/SONAR",
        languages=["sv"],
        open_source=True,
        embedding_size=1024,
        model_type="SONAR",
        release_date=date(2023, 5, 17),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sonar_model, source_lang="swe_Latn")),
        meta=meta,
    )


@models.register("sonar-nob")
def create_sonar_nb() -> SebModel:
    meta = ModelMeta(
        name="sonar-nob",
        huggingface_name=None,
        reference="https://github.com/facebookresearch/SONAR",
        languages=["nb"],
        open_source=True,
        embedding_size=1024,
        model_type="SONAR",
        release_date=date(2023, 5, 17),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sonar_model, source_lang="nob_Latn")),
        meta=meta,
    )


@models.register("sonar-nno")
def create_sonar_nn() -> SebModel:
    meta = ModelMeta(
        name="sonar-nno",
        huggingface_name=None,
        reference="https://github.com/facebookresearch/SONAR",
        languages=["nn"],
        open_source=True,
        embedding_size=1024,
        release_date=date(2023, 5, 17),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sonar_model, source_lang="nno_Latn")),
        meta=meta,
    )



from seb.interfaces.task import Task
from seb.registries import tasks

from ..interfaces.mteb_task import MTEBTask


@tasks.register("Angry Tweets")
def create_angry_tweets() -> Task:
    from mteb import AngryTweetsClassification

    task = MTEBTask(AngryTweetsClassification())
    task.name = "Angry Tweets"
    task.domain = ["social"]
    return task


@tasks.register("LCC")
def create_lcc() -> Task:
    from mteb import LccSentimentClassification

    task = MTEBTask(LccSentimentClassification())
    task.name = "LCC"  # type: ignore
    task.domain = [
        "legal",
        "web",
        "news",
        "social",
        "fiction",
        "non-fiction",
        "academic",
        "government",
    ]
    return task


@tasks.register("Bornholm Parallel")
def create_bornholm_parallel() -> Task:
    from mteb import BornholmBitextMining

    task = MTEBTask(BornholmBitextMining())
    task.name = "Bornholm Parallel"
    task.domain = ["poetry", "wiki", "fiction", "web", "social"]
    task._text_columns = ["sentence1", "sentence2"]
    return task


@tasks.register("DKHate")
def create_dkhate() -> Task:
    from mteb import DKHateClassification

    task = MTEBTask(DKHateClassification())
    task.name = "DKHate"
    task.domain = ["social"]
    return task


@tasks.register("Da Political Comments")
def create_da_political_comments() -> Task:
    from mteb import DanishPoliticalCommentsClassification

    task = MTEBTask(DanishPoliticalCommentsClassification())
    task.name = "Da Political Comments"
    task.domain = ["social"]
    task.reference = "https://huggingface.co/datasets/danish_political_comments"  # TODO: Make a PR for MTEB to add this reference
    return task


@tasks.register("DanFEVER")
def create_dan_fever() -> Task:
    from seb.mteb_tasks import DanFever

    task = MTEBTask(DanFever())
    task.name = "DanFEVER"
    task.domain = ["wiki", "non-fiction"]
    return task


@tasks.register("TV2Nord Retrieval")
def create_tv2nord_retrieval() -> Task:
    from seb.mteb_tasks import TV2Nordretrieval

    task = MTEBTask(TV2Nordretrieval())
    task.name = "TV2Nord Retrieval"
    task.domain = ["news", "non-fiction"]
    return task


@tasks.register("Twitterhjerne")
def create_twitterhjerne() -> Task:
    from seb.mteb_tasks import TwitterHjerneRetrieval

    task = MTEBTask(TwitterHjerneRetrieval())
    task.name = "Twitterhjerne"
    task.domain = ["social"]
    return task



import os


class WarningIgnoreContextManager:
    """
    A context manager for ignoring warnings. When running a task
    """

    original_states = {}  # noqa: RUF012

    def __enter__(self):
        self.ignore_tokenizers_parallelism_warning()
        self.ignore_convergence_warnings()

    def __exit__(self, exc_type, exc_value, traceback):  # noqa: ANN001
        self.restore_original_states()

    def restore_original_states(self) -> None:
        for key, value in self.original_states.items():
            if value is None:
                del os.environ[key]
            else:
                os.environ[key] = value

    def ignore_tokenizers_parallelism_warning(self) -> None:
        """
        Ignore warnings from HuggingFace tokenizers casuses by multiple threads.
        """
        tok_parallel = os.environ.get("TOKENIZERS_PARALLELISM", None)
        self.original_states["TOKENIZERS_PARALLELISM"] = tok_parallel
        if tok_parallel is None:
            os.environ["TOKENIZERS_PARALLELISM"] = "false"

    def ignore_convergence_warnings(self) -> None:
        """
        Ignore convergence warnings from sklearn.
        """
        p_warnings = os.environ.get("PYTHONWARNINGS", None)
        self.original_states["PYTHONWARNINGS"] = p_warnings
        os.environ["PYTHONWARNINGS"] = "ignore::UserWarning"



import logging
from functools import partial
from pathlib import Path
from typing import Literal, Optional

from radicli import Arg, get_list_converter

from seb.registered_models.hf_models import get_sentence_transformer

import seb
from seb.registered_models.hf_models import SentenceTransformerWithTaskEncode
from seb.registries import get_all_models

from .cli import cli
from .import_code import import_code
from .table import convert_to_table, pretty_print_benchmark

logger = logging.getLogger(__name__)


def build_model(model_name: str) -> seb.SebModel:
    all_models = seb.models.get_all().keys()

    if model_name in seb.models:
        logger.info("Model registered in SEB. Loading from registry.")
        return seb.models.get(model_name)()

    logger.info("Model not found in SEB registry. Wrapping using setenceTransformers.")
    logger.debug(f"Models in registries include include: {all_models}")
    meta = seb.ModelMeta(
        name=Path(model_name).stem,
    )
    model = seb.SebModel(
        meta=meta,
        encoder=seb.LazyLoadEncoder(partial(get_sentence_transformer, model_name=model_name)),  # type: ignore
    )
    return model


def dump_results(results: list[seb.BenchmarkResults], output_path: Path):
    for result in results:
        mdl_path_name = result.meta.get_path_name()
        result.to_disk(output_path / mdl_path_name)


@cli.command(
    "run",
    models=Arg(
        "--models",
        "-m",
        help="Model names or paths."
        " If a model is not registrered in SEB it will be loaded using SentenceTransformers."
        " If none are specified the whole benchmark is run.",
        converter=get_list_converter(str, delimiter=","),
    ),
    output_path=Arg(
        "--output-path",
        "-o",
        help="Directory to save all results to.",
    ),
    languages=Arg(
        "--languages",
        "-l",
        help="What languages subsection to run the benchmark on. If left blank it will run it on all languages.",
        converter=get_list_converter(str, delimiter=","),
    ),
    tasks=Arg(
        "--tasks",
        "-t",
        help="What tasks should model be run on. Default to all tasks within the specified languages.",
        converter=get_list_converter(str, delimiter=","),
    ),
    ignore_cache=Arg(
        "--ignore-cache",
        help="Ignores caches models. Note that SEB ships with an existing cache. You can set the cache_dir using the environmental variable SEB_CACHE_DIR",
    ),
    ignore_errors=Arg(
        "--ignore-errors",
        help="Should errors be ignored when running a model on a benchmark task.",
    ),
    code_path=Arg(
        "--code",
        "-c",
        help="Code to run before executing benchmark. Useful for adding custom model to registries.",
    ),
    logging_level=Arg("--logging-level", help="Logging level for the benchmark."),
)
def run_benchmark_cli(
    models: Optional[list[str]] = None,
    output_path: Optional[Path] = None,
    tasks: Optional[list[str]] = None,
    languages: Optional[list[str]] = None,
    ignore_cache: bool = False,
    ignore_errors: bool = False,
    code_path: Optional[Path] = None,
    logging_level: Literal["DEBUG", "INFO"] = "INFO",
) -> None:
    """
    Runs the Benchmark either on specified models or on all registered models.
    Can save the benchmark's results, but also displays them in a table similar
    to the official website.

    **Examples:**

    To run all models on all languages and tasks:
    ```{bash}
    seb run
    ```

    To run a model on all languages and tasks:
    ```{bash}
    seb run -m sentence-transformers/all-MiniLM-L6-v2
    ```

    To run multiple models:
    To run a model on all languages and tasks:
    ```{bash}
    seb run -m sentence-transformers/all-MiniLM-L6-v2,sentence-transformers/all-mpnet-base-v2
    ```

    if you only want to limit it to a subset of languages or tasks you can use the `--languages` and `--tasks` flags.
    ```{bash}
    # Running a model on a subset of languages
    seb run sentence-transformers/all-MiniLM-L6-v2 -o results/ -l nb,nn

    # Running a model on a subset of tasks
    seb run sentence-transformers/all-MiniLM-L6-v2 -o results/ -t DKHate,ScaLA
    ```

    """
    logging.basicConfig(level=logging_level)

    import_code(code_path)

    benchmark = seb.Benchmark(languages, tasks=tasks)
    if models is None:  # noqa
        emb_models = get_all_models()
    else:
        emb_models = [build_model(model_name=model_name) for model_name in models]
    benchmark_results = benchmark.evaluate_models(
        emb_models,
        use_cache=not ignore_cache,
        raise_errors=not ignore_errors,
        run_model=True,
    )

    if output_path is not None:
        output_path.mkdir(exist_ok=True)
        dump_results(benchmark_results, output_path)

    highlight = [model.meta.name for model in emb_models]

    # Dummy run all models for the sake of printing the table
    current_models = {mdl.meta.name for mdl in emb_models}
    for _, create_mdl in seb.models.get_all().items():
        mdl = create_mdl()
        if mdl.meta.name not in current_models:
            emb_models.append(mdl)

    benchmark = seb.Benchmark(languages, tasks=tasks)
    benchmark_results = benchmark.evaluate_models(
        emb_models,
        use_cache=True,
        raise_errors=False,
        run_model=False,
    )

    new_highlight_names = []
    for model in benchmark_results:
        if model.meta.name in highlight:
            model.meta.name = f"NEW: {model.meta.name}"
            new_highlight_names.append(model.meta.name)
    benchmark_df = convert_to_table(benchmark_results, languages)
    pretty_print_benchmark(benchmark_df, highlight=new_highlight_names)



"""
Notes:

The dataset is quite small only containing 78 queries and 262 documents.
"""

from typing import Any

import datasets
from mteb.abstasks import AbsTaskRetrieval


class TwitterHjerneRetrieval(AbsTaskRetrieval):
    @property
    def description(self) -> dict[str, Any]:
        return {
            "name": "TwitterHjerneRetrieval",
            "hf_hub_name": "sorenmulli/da-hashtag-twitterhjerne",
            "description": "Danish question asked on Twitter with the Hashtag #Twitterhjerne ('Twitter brain') and their corresponding answer.",
            "reference": "https://huggingface.co/datasets/sorenmulli/da-hashtag-twitterhjerne",
            "type": "Retrieval",
            "category": "p2p",
            "eval_splits": ["train"],
            "eval_langs": ["da"],
            "main_score": "ndcg_at_10",
            "revision": "099ee143c7fdfa6bd7965be8c801cb161c313b29",
        }

    def load_data(self, **kwargs: dict):  # noqa: ARG002
        """
        Load dataset from HuggingFace hub
        """
        if self.data_loaded:
            return

        self.dataset: datasets.DatasetDict = datasets.load_dataset(
            self.description["hf_hub_name"],
            revision=self.description.get("revision"),
        )  # type: ignore

        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self) -> None:
        """
        and transform to a retrieval datset, which have the following attributes

        self.corpus = Dict[doc_id, Dict[str, str]] #id => dict with document datas like title and text
        self.queries = Dict[query_id, str] #id => query
        self.relevant_docs = Dict[query_id, Dict[[doc_id, score]]
        """
        self.corpus = {}
        self.relevant_docs = {}
        self.queries = {}
        text2id = {}

        for split in self.dataset:
            ds: datasets.Dataset = self.dataset[split]  # type: ignore
            ds = ds.map(answers_to_list)

            self.queries[split] = {}
            self.relevant_docs[split] = {}
            self.corpus[split] = {}

            questions = ds["Question"]
            answers = ds["answers"]

            n = 0
            for q, answ in zip(questions, answers):
                if len(q.split(" ")) < 4 and answ:
                    continue
                query_id = str(n)
                self.queries[split][query_id] = q
                n += 1
                answer_ids = []
                for a in answ:
                    if a not in text2id:
                        text2id[a] = n
                        answer_id = str(n)
                        self.corpus[split][answer_id] = {"title": "", "text": a}
                        n += 1
                    else:
                        answer_id = str(text2id[a])
                    answer_ids.append(answer_id)

                self.relevant_docs[split][query_id] = {answer_id: 1 for answer_id in answer_ids}


def answers_to_list(example: dict) -> dict:
    example["answers"] = [v for k, v in example.items() if k.startswith("Answer") and v and len(v.split(" ")) > 3]
    return example



"""
All the models registered in the benchmark, along with their metadata.
"""
import logging
from datetime import date
from functools import partial
from typing import Any, Optional

from numpy.typing import ArrayLike
from sentence_transformers import SentenceTransformer

from seb.interfaces.model import LazyLoadEncoder, ModelMeta, SebModel
from seb.interfaces.task import Task
from seb.registries import models


def silence_warnings_from_sentence_transformers():
    from sentence_transformers.SentenceTransformer import logger

    logger.setLevel(logging.ERROR)


class SentenceTransformerWithTaskEncode(SentenceTransformer):
    """
    A sentence transformer wrapper that allows for encoding with a task.
    """

    def encode(  # type: ignore
        self,
        sentences: list[str],
        *,
        batch_size: int,
        task: Optional[Task] = None,  # noqa: ARG002
        **kwargs: Any,
    ) -> ArrayLike:
        return super().encode(sentences, batch_size=batch_size, **kwargs)  # type: ignore


def get_sentence_transformer(
    model_name: str,
    max_seq_length: Optional[int] = None,
) -> SentenceTransformerWithTaskEncode:
    silence_warnings_from_sentence_transformers()
    mdl = SentenceTransformerWithTaskEncode(model_name)
    if max_seq_length is not None:
        mdl.max_seq_length = max_seq_length
    return mdl


@models.register("jinaai/jina-embedding-b-en-v1")
def create_jina_base() -> SebModel:
    hf_name = "jinaai/jina-embedding-b-en-v1"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["en"],
        open_source=True,
        embedding_size=768,
        model_type="T5",
        release_date=date(2023, 7, 7),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


# Relevant multilingual models
@models.register("sentence-transformers/all-MiniLM-L6-v2")
def create_all_mini_lm_l6_v2() -> SebModel:
    hf_name = "sentence-transformers/all-MiniLM-L6-v2"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["en"],
        open_source=True,
        embedding_size=384,
        model_type="BERT",
        release_date=date(2021, 6, 30),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2")
def create_multilingual_mini_lm_l12_v2() -> SebModel:
    hf_name = "sentence-transformers/paraphrase-multilingual-MiniLM-L12-v2"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=[],
        open_source=True,
        embedding_size=384,
        model_type="BERT",
        release_date=date(2021, 6, 2),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("sentence-transformers/paraphrase-multilingual-mpnet-base-v2")
def create_multilingual_mpnet_base_v2() -> SebModel:
    hf_name = "sentence-transformers/paraphrase-multilingual-mpnet-base-v2"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=[],
        open_source=True,
        embedding_size=768,
        model_type="BERT",
        release_date=date(2021, 6, 2),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("KBLab/sentence-bert-swedish-cased")
def create_sentence_swedish_cased() -> SebModel:
    hf_name = "KBLab/sentence-bert-swedish-cased"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["sv"],
        open_source=True,
        embedding_size=768,
        model_type="BERT",
        release_date=date(2021, 8, 8),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("jonfd/electra-small-nordic")
def create_electra_small_nordic() -> SebModel:
    hf_name = "jonfd/electra-small-nordic"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["da", "nb", "sv", "nn"],
        open_source=True,
        embedding_size=256,
        model_type="ELECTRA",
        release_date=date(2022, 1, 31),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("vesteinn/DanskBERT")
def create_dansk_bert() -> SebModel:
    hf_name = "vesteinn/DanskBERT"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["da"],
        open_source=True,
        embedding_size=768,
        model_type="XLM-R",
        release_date=date(2022, 11, 23),
    )

    return SebModel(
        # see https://huggingface.co/vesteinn/DanskBERT/discussions/3
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name, max_seq_length=512)),  # type: ignore
        meta=meta,
    )


@models.register("chcaa/dfm-encoder-large-v1")
def create_dfm_encoder_large_v1() -> SebModel:
    hf_name = "chcaa/dfm-encoder-large-v1"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["da"],
        open_source=True,
        embedding_size=1024,
        release_date=date(2023, 1, 4),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("NbAiLab/nb-bert-large")
def create_nb_bert_large() -> SebModel:
    hf_name = "NbAiLab/nb-bert-large"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["nb", "nn"],
        open_source=True,
        embedding_size=1024,
        model_type="BERT",
        release_date=date(2021, 4, 29),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("NbAiLab/nb-bert-base")
def create_nb_bert_base() -> SebModel:
    hf_name = "NbAiLab/nb-bert-base"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["nb", "nn"],
        open_source=True,
        embedding_size=768,
        model_type="BERT",
        release_date=date(2021, 1, 13),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


"""
# excluded due to loading issues: https://huggingface.co/ltg/norbert3-base/discussions/1
# I could create a custom encoder for this model

@models.register("ltg/norbert3-large")
def create_norbert3_large() -> odel:
    hf_name = "ltg/norbert3-large"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["nb", "nn"],
    )
    return odel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name),  # type: ignore
        meta=meta,
    )


@models.register("ltg/norbert3-base")
def create_norbert3_base() -> odel:
    hf_name = "ltg/norbert3-base"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["nb", "nn"],
    )
    return odel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name),  # type: ignore
        meta=meta,
    )
"""


@models.register("KB/bert-base-swedish-cased")
def create_bert_base_swedish_cased() -> SebModel:
    hf_name = "KB/bert-base-swedish-cased"

    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["sv"],
        open_source=True,
        embedding_size=768,
        model_type="BERT",
        release_date=date(2022, 6, 7),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name, max_seq_length=512)),  # type: ignore
        meta=meta,
    )


@models.register("kb/electra-small-swedish-cased-discriminator")
def create_electra_small_swedish_cased_discriminator() -> SebModel:
    hf_name = "kb/electra-small-swedish-cased-discriminator"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["sv"],
        open_source=True,
        embedding_size=256,
        model_type="ELECTRA",
        release_date=date(2022, 6, 7),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


# Multilingual baselines
@models.register("xlm-roberta-base")
def create_xlm_roberta_base() -> SebModel:
    hf_name = "xlm-roberta-base"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        open_source=True,
        embedding_size=768,
        release_date=date(2019, 6, 29),
    )

    # Beware that this uses mean pooling currently, and we might want to change it to CLS in the future
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name, max_seq_length=512)),  # type: ignore
        meta=meta,
    )


@models.register("xlm-roberta-large")
def create_xlm_roberta_large() -> SebModel:
    hf_name = "xlm-roberta-large"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        open_source=True,
        embedding_size=1024,
        model_type="XLM-R",
    )

    # Beware that this uses mean pooling currently, and we might want to change it to CLS in the future
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("sentence-transformers/LaBSE")
def create_labse() -> SebModel:
    hf_name = "sentence-transformers/LaBSE"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        open_source=True,
        embedding_size=768,
        model_type="BERT",
        release_date=date(2020, 10, 12),
    )

    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


# Scandinavian sentence encoders
@models.register("KennethEnevoldsen/dfm-sentence-encoder-large-1")
def create_dfm_sentence_encoder_large() -> SebModel:
    hf_name = "KennethEnevoldsen/dfm-sentence-encoder-large-1"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["da"],
        open_source=True,
        embedding_size=1024,
        model_type="BERT",
        release_date=date(2023, 11, 15),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("KennethEnevoldsen/dfm-sentence-encoder-large-exp1")
def create_dfm_sentence_encoder_large_exp() -> SebModel:
    hf_name = "KennethEnevoldsen/dfm-sentence-encoder-large-exp1"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["da"],
        open_source=True,
        embedding_size=1024,
        release_date=date(2023, 11, 15),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("KennethEnevoldsen/dfm-sentence-encoder-small-v1")
def create_dfm_sentence_encoder_small() -> SebModel:
    hf_name = "KennethEnevoldsen/dfm-sentence-encoder-small-v1"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["da"],
        open_source=True,
        embedding_size=256,
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("KennethEnevoldsen/dfm-sentence-encoder-medium-v1")
def create_dfm_sentence_encoder_medium() -> SebModel:
    hf_name = "KennethEnevoldsen/dfm-sentence-encoder-medium-v1"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["da"],
        open_source=True,
        embedding_size=768,
        model_type="BERT",
        release_date=date(2023, 11, 15),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )


@models.register("KennethEnevoldsen/dfm-sentence-encoder-large-exp2-no-lang-align")
def create_dfm_sentence_encoder_large_exp2() -> SebModel:
    hf_name = "KennethEnevoldsen/dfm-sentence-encoder-large-exp2-no-lang-align"
    meta = ModelMeta(
        name=hf_name.split("/")[-1],
        huggingface_name=hf_name,
        reference=f"https://huggingface.co/{hf_name}",
        languages=["da"],
        open_source=True,
        embedding_size=1024,
        model_type="BERT",
        release_date=date(2023, 11, 15),
    )
    return SebModel(
        encoder=LazyLoadEncoder(partial(get_sentence_transformer, model_name=hf_name)),  # type: ignore
        meta=meta,
    )
